#!/bin/sh

if [ -z "$1" ] ; then
    echo "Syntax: $0 (-f file | -b) [seed [seed ...]]"
    echo "-f uses the given file for the data"
    echo "-b uses a built-in test"
    echo "The remaining arguments are considered random seeds. If no argument is provided, a random seed will be chosen."
    exit 0
fi

if [ "$1" = "-f" ] ; then
    SUFFIX="_data"
    shift
    if [ ! -r "$1" ] ; then
        echo "Invalid or missing file $1"
        # no argument => call help
        exec $0
    fi
    FILE=$1
    echo "Processing experiments on file $FILE"
elif [ "$1" = "-b" ] ; then
    echo "Processing built-in test"
else
    # no argument => call help
    exec $0
fi

shift

if [ -n "$1" ] ; then
    SEED=$*
else
    SEED=$RANDOM
fi

for s in $SEED ; do

echo "Processing base experiment for seed = $s"
nice -n 19 ./base_experiment $s $FILE >& log_base${SUFFIX}_${s}.txt

echo "Processing hebbian experiment for seed = $s"
nice -n 19 ./hebbian_experiment $s $FILE >& log_hebbian${SUFFIX}_${s}.txt

echo "Processing multifractal experiment for seed = $s"
nice -n 19 ./multifractal_experiment $s $FILE >& log_multifractal${SUFFIX}_${s}.txt

done
