
dnl PKG_CHECK_MODULES(GSTUFF, gtk+-2.0 >= 1.3 glib = 1.3.4, action-if, action-not)
dnl defines GSTUFF_LIBS, GSTUFF_CFLAGS, see pkg-config man page
dnl also defines GSTUFF_PKG_ERRORS on error
AC_DEFUN([PKG_CHECK_MODULES], [
  succeeded=no

  if test -z "$PKG_CONFIG"; then
    AC_PATH_PROG(PKG_CONFIG, pkg-config, no)
  fi

  if test "$PKG_CONFIG" = "no" ; then
     echo "*** The pkg-config script could not be found. Make sure it is"
     echo "*** in your path, or set the PKG_CONFIG environment variable"
     echo "*** to the full path to pkg-config."
     echo "*** Or see http://www.freedesktop.org/software/pkgconfig to get pkg-config."
  else
     PKG_CONFIG_MIN_VERSION=0.9.0
     if $PKG_CONFIG --atleast-pkgconfig-version $PKG_CONFIG_MIN_VERSION; then
        AC_MSG_CHECKING(for $2)

        if $PKG_CONFIG --exists "$2" ; then
            AC_MSG_RESULT(yes)
            succeeded=yes

            AC_MSG_CHECKING($1_CFLAGS)
            $1_CFLAGS=`$PKG_CONFIG --cflags "$2"`
            AC_MSG_RESULT($$1_CFLAGS)

            AC_MSG_CHECKING($1_LIBS)
            $1_LIBS=`$PKG_CONFIG --libs "$2"`
            AC_MSG_RESULT($$1_LIBS)
        else
            $1_CFLAGS=""
            $1_LIBS=""
            ## If we have a custom action on failure, don't print errors, but 
            ## do set a variable so people can do so.
            $1_PKG_ERRORS=`$PKG_CONFIG --errors-to-stdout --print-errors "$2"`
            ifelse([$4], ,echo $$1_PKG_ERRORS,)
        fi

        AC_SUBST($1_CFLAGS)
        AC_SUBST($1_LIBS)
     else
        echo "*** Your version of pkg-config is too old. You need version $PKG_CONFIG_MIN_VERSION or newer."
        echo "*** See http://www.freedesktop.org/software/pkgconfig"
     fi
  fi

  if test $succeeded = yes; then
     ifelse([$3], , :, [$3])
  else
     ifelse([$4], , AC_MSG_ERROR([Library requirements ($2) not met; consider adjusting the PKG_CONFIG_PATH environment variable if your libraries are in a nonstandard prefix so pkg-config can find them.]), [$4])
  fi
])




AC_DEFUN(AMYG_ADD_DEPENDENCIES,
[
   [A]M_DEPENDENCIES(CC)
   [A]M_DEPENDENCIES(CXX)
])

AC_DEFUN(AMYG_CHECK_COMPILER_FLAG,
[
    dnl AC_REQUIRE([AC_CHECK_COMPILERS]) <- breaks with autoconf 2.50
    AC_MSG_CHECKING(whether $CXX supports -$1)
    amyg_cache=`echo $1 | sed 'y%.=/+-%___p_%'`
    AC_CACHE_VAL(amyg_cv_prog_cxx_$amyg_cache,
    [
        echo 'int main() { return 0; }' >conftest.cc
        eval "amyg_cv_prog_cxx_$amyg_cache=no"
        if test -z "`$CXX -$1 -c conftest.cc 2>&1`"; then
            if test -z "`$CXX -$1 -o conftest conftest.o 2>&1`"; then
                eval "amyg_cv_prog_cxx_$amyg_cache=yes"
            fi
        fi
        rm -f conftest*
    ])
    if eval "test \"`echo '$amyg_cv_prog_cxx_'$amyg_cache`\" = yes"; then
        AC_MSG_RESULT(yes)
        :
        $2
    else
        AC_MSG_RESULT(no)
        :
        $3
    fi
])

AC_DEFUN(AC_CHECK_COMPILERS,
[
  AC_ARG_ENABLE(debug,[  --enable-debug          enables debug symbols [default=no]],
  [
   if test $enableval = "no"; dnl
     then
       amyg_use_debug_code="no"
       amyg_use_debug_define="yes"
     else
       amyg_use_debug_code="yes"
       amyg_use_debug_define="no"
   fi
  ],
    [amyg_use_debug_code="no"
      amyg_use_debug_define="no"
  ])

  dnl Just for configure --help
  AC_ARG_ENABLE(dummyoption,[  --disable-debug         disables debug output and debug symbols [default=no]],[],[])

  dnl enable-strict is not yet implemented, but left in place for future (possible) development
  AC_ARG_ENABLE(strict,[  --enable-strict         compiles with strict compiler options (may not work!)],
   [
    if test $enableval = "no"; then
         amyg_use_strict_options="no"
       else
         amyg_use_strict_options="yes"
    fi
   ], [amyg_use_strict_options="no"])

  AC_ARG_ENABLE(profile,[  --enable-profile        creates profiling infos [default=no]],
    [amyg_use_profiling=$enableval],
    [amyg_use_profiling="no"]
  )

  AC_ARG_ENABLE(logging,[  --enable-logging=LOGLEVEL         [default=no]],
    [amyg_use_logging=$enableval],
    [amyg_use_logging="no"]
  )

  AC_ARG_ENABLE(optimize,[  --enable-optimize       enables optimization flag [default=no]],
    [amyg_use_optimize=$enableval],
    [amyg_use_optimize="no"]
  )

  AC_ARG_ENABLE(expensive,[  --enable-expensive      enables expensive optimizations [default=no]],
    [amyg_use_expensive=$enableval],
    [amyg_use_expensive="no"]
  )

  AC_ARG_ENABLE(march,[  --enable-march          enables the march compiler parameter],
    [amyg_arch=$enableval],
    [amyg_arch="none"]
  )

  dnl this prevents stupid AC_PROG_CC to add "-g" to the default CFLAGS
  CFLAGS=" $CFLAGS"

  AC_PROG_CC

  if test "$GCC" = "yes"; then
    if test "$amyg_use_debug_code" = "yes"; then
      CFLAGS="-g $CFLAGS"
      case $host in
        *-*-linux-gnu)
          CFLAGS="-ansi -W -Wall -Wshadow -Wpointer-arith -Wmissing-prototypes -Wwrite-strings -D_XOPEN_SOURCE=500 -D_BSD_SOURCE $CFLAGS"
        ;;
      esac
    else
      CFLAGS="-O2 $CFLAGS"
    fi
  fi

  if test "$amyg_use_debug_define" = "yes"; then
    CFLAGS="-DNDEBUG $CFLAGS"
  fi

  case "$host" in
  *-*-sysv4.2uw*) CFLAGS="-D_UNIXWARE $CFLAGS";;
  *-*-sysv5uw7*) CFLAGS="-D_UNIXWARE7 $CFLAGS";;
  esac

  if test -z "$LDFLAGS" && test "$amyg_use_debug_code" = "no" && test "$GCC" = "yes"; then
     LDFLAGS=""
  fi

  CXXFLAGS=" $CXXFLAGS"

    case "$host" in
    *-*-cygwin*)
        dnl echo "Setting ldl for cygwin"
        LDFLAGS=" $LDFLAGS -lltdl"
    ;;
    *)
        LDFLAGS=" $LDFLAGS -lltdl"
    ;;
    esac

  AC_PROG_CXX

  if test "$GXX" = "yes"; then
    if test "$amyg_use_debug_code" = "yes"; then
      CXXFLAGS="-Wall -W -Wpointer-arith -Wmissing-prototypes -Wwrite-strings $CXXFLAGS -I.. -I."
      if test "$amyg_use_optimize" = "yes"; then
          CXXFLAGS="-g2 -O2 $CXXFLAGS"
      else
          CXXFLAGS="-g2 $CXXFLAGS"
      fi

      dnl AMYG_CHECK_COMPILER_FLAG(Wno-long-long,[CXXFLAGS="-Wno-long-long $CXXFLAGS"])
      dnl AMYG_CHECK_COMPILER_FLAG(Wnon-virtual-dtor,[CXXFLAGS="-Wnon-virtual-dtor $CXXFLAGS"])
      dnl AMYG_CHECK_COMPILER_FLAG(fno-builtin,[CXXFLAGS="-fno-builtin $CXXFLAGS"])

      case $host in  dnl
      *-*-linux-gnu)
        CXXFLAGS="-ansi -D_XOPEN_SOURCE=500 -D_BSD_SOURCE -Wbad-function-cast -Wcast-align -Wundef -Wconversion $CXXFLAGS"
        ;;
      esac

      if test "$amyg_use_strict_options" = "yes"; then
        CXXFLAGS="-pedantic -Wcast-qual -Wbad-function-cast -Wshadow -Wcast-align $CXXFLAGS"
      fi

      if test "$amyg_very_strict" = "yes"; then
        CXXFLAGS="-Wold-style-cast -Wredundant-decls -Wconversion $CXXFLAGS"
      fi
    else
        if test "$amyg_use_expensive" = "yes"; then
            AMYG_CHECK_COMPILER_FLAG(fexpensive-optimizations, [CXXFLAGS="-fexpensive-optimizations $CXXFLAGS"])
        fi
	if test "$amyg_arch" != "none"; then
		CXXFLAGS="-march=$amyg_arch $CXXFLAGS"
	fi
        CXXFLAGS="-O3 -Wall $CXXFLAGS"
    fi
    AMYG_CHECK_COMPILER_FLAG(D_REENTRANT, [CXXFLAGS="-D_REENTRANT $CXXFLAGS"])
    AMYG_CHECK_COMPILER_FLAG(pthread, [CXXFLAGS="-pthread $CXXFLAGS"])
  fi

  if test "$amyg_use_debug_define" = "yes"; then
    CXXFLAGS="-DNDEBUG $CXXFLAGS"
  fi

  if test "$amyg_use_profiling" = "yes"; then
    AMYG_CHECK_COMPILER_FLAG(pg,
    [
      CFLAGS="-pg $CFLAGS"
      CXXFLAGS="-pg -DPROFILING $CXXFLAGS"
    ])
  fi

  if test "$amyg_use_logging" != "no"; then
      CFLAGS="-DLOGLEVEL=${amyg_use_logging} $CFLAGS"
      CXXFLAGS="-DLOGLEVEL=${amyg_use_logging} $CXXFLAGS"
  fi


dnl  AMYG_CHECK_COMPILER_FLAG(fno-exceptions,[CXXFLAGS="$CXXFLAGS -fno-exceptions"])
dnl  AMYG_CHECK_COMPILER_FLAG(fno-check-new, [CXXFLAGS="$CXXFLAGS -fno-check-new"])
dnl  AMYG_CHECK_COMPILER_FLAG(fexceptions, [USE_EXCEPTIONS="-fexceptions"], USE_EXCEPTIONS=	)
  AC_SUBST(USE_EXCEPTIONS)
  dnl obsolete macro - provided to keep things going
  USE_RTTI=
  AC_SUBST(USE_RTTI)

  case "$host" in
      *-*-irix*)  test "$GXX" = yes && CXXFLAGS="-D_LANGUAGE_C_PLUS_PLUS -D__LANGUAGE_C_PLUS_PLUS $CXXFLAGS" ;;
      *-*-sysv4.2uw*) CXXFLAGS="-D_UNIXWARE $CXXFLAGS";;
      *-*-sysv5uw7*) CXXFLAGS="-D_UNIXWARE7 $CXXFLAGS";;
  esac


  AC_PROG_CXXCPP

  AC_SUBST(NOOPT_CXXFLAGS)

  ifdef([AM_DEPENDENCIES], AC_REQUIRE([AMYG_ADD_DEPENDENCIES]), [])

  AMYG_CXXFLAGS=
  AC_SUBST(AMYG_CXXFLAGS)
])

AC_DEFUN(AC_SET_XML_FLAGS,
[
    CXXFLAGS="$CXXFLAGS $XML_CFLAGS"
    dnl LIBS="$XML_LIBS $LIBS"
])

AC_DEFUN(AC_GUESS_XML_FLAGS,
[
    if test "$XML2_CONFIG" = "no" ; then
        echo "Guessing libxml2 config values."
        echo "libxml2 include dir = /usr/include/libxml2"
        CXXFLAGS="$CXXFLAGS -I/usr/inlcude/libxml2 -I/usr/include/libxml2/libxml"
        XML_LIBS="-lxml2 -lz -lm"
    else
        AC_MSG_ERROR([libxml2 configure failed])
    fi
])


dnl ----------------------------------------------------------------------
