/***************************************************************************
                          amygdalaclass.h  -  description
                             -------------------
    begin                : Wed Jul 2 2003
    copyright            : (C) 2003 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef AMYGDALACLASS_H
#define AMYGDALACLASS_H

#include <string>
#include <cxxabi.h>

#define GCC_VERSION (__GNUC__ * 10000 \
                     + __GNUC_MINOR__ * 100 \
                     + __GNUC_PATCHLEVEL__)


/** @mainpage Amygdala API documentation
 *
 * This is a fairly complete API documentation about all classes and their public
 * and protected members. This doesn't mean that you need everything documented in
 * here. In fact you'll only use a small part of this directly. Please see the
 * examples in the samples/ directory to get started.
 * <h2>Please contribute</h2>
 * Amygdala is an OpenSource project which is not affiliated to any institution.
 * We're a loosely knit group of developers aiming to create a flexible, 
 * user friendly and powerful simulator for spiking neural networks. If you're
 * experienced in C++, neural networks, OpenGL, QT or just interested in
 * this exciting way of doing computation, please contribute. 
 */



/** @brief The namespace for all of Amygdala
  */

namespace Amygdala {
// Basic data types.  These can be sized according to machine architecture
// and specific simulation requirements.
// TODO: Allow these to be sized as unsigned long or unsigned long long
// according to configure script options.

/** datatype used for simulation time */
typedef unsigned long long AmTimeInt;
/** datatype used for classes that need an ID, mainly Neuron */
typedef unsigned long long AmIdInt;
/** datatype for group IDs 
  * @see OutputGroup
  */
typedef unsigned long AmGroupInt;

/** @class AmygdalaClass amygdalaclass.h amygdala/amygdalaclass.h
 * @brief The base class for all Amygdala classes.
 *
 * AmygdalaClass provides basic utility functions and types that
 * are needed by all classes in the Amygdala namespace.
 * @author Matt Grover
 * @author Rudiger Koch
 */
class AmygdalaClass {
public:
    AmygdalaClass() {};
    virtual ~AmygdalaClass() = 0;


/** @return The demangled class name.
  * @param fullyQualified give the fully qualified Classname such as
  * Amygdala::Genome if true else return only the classname
  */
    std::string GetClassName(bool fullyQualified=false);
protected:
};
} // namespace Amygdala


#endif

#undef GetClassName // This is a macro in MessySoft Windoze

