/***************************************************************************
                          flatlayer.h  -  description
                             -------------------
    copyright            : (C) 2004 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef FLATLAYER_H
#define FLATLAYER_H

#include <amygdala/topology.h>
#include <amygdala/factory.h>
#include <string>

namespace Amygdala {
	
/** @class FlatLayer flatlayer.h amygdala/flatlayer.h
 * FlatLayer is a simple Topology class with no restrictions on how member neurons
 * can connect to each other or neurons within other Topology objects.
 * FlatLayer can be used as a general purpose neuron container.
 * @see Topology
 * @author Matt Grover <mgrover@amygdala.org> */
class FlatLayer: public Topology {
public:
    typedef TopologyFactory<FlatLayer> Factory;
    virtual ~FlatLayer() {}
protected:
    FlatLayer(std::string& name):Topology(name) {};
    /** Do any class-specific initialization after one of the MakeNeuron() functions has been called. */
    virtual void InitNewNeuron(Neuron* nrn);
private:
    friend class Network;
    template<class topology>
    friend class TopologyFactory;
};

namespace Factory {
    static FlatLayer::Factory MakeFlatLayer;
}
}   // namespace Amygdala

#endif
