/***************************************************************************
                          netloader.cpp  -  description
                             -------------------
    begin                : Mon Apr 29 2002
    copyright            : (C) 2002 by Rudiger Koch
    email                : rkoch@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "amygdala/netloader.h"
#include "amygdala/synapse.h"
#include "amygdala/synapseproperties.h"
#include "amygdala/neuron.h"
#include "amygdala/inputneuron.h"
#include "amygdala/basicneuron.h"
#include "amygdala/utilities.h"
#include "amygdala/axon.h"
#include "amygdala/axonnode.h"
#include "amygdala/physicalproperties.h"
#include "amygdala/spikeinput.h"
#include "amygdala/spikeoutput.h"
#include "amygdala/outputmanager.h"
#include "amygdala/nconnector.h"

#include <stdexcept>
#include <iostream>

#include <libxml/parser.h>
#include <libxml/parserInternals.h>

using namespace std;
using namespace Amygdala;


extern "C" {
    void __SAXStartElement1(NetLoader *netLoader, const char *name, const char **attrs){
        netLoader->SAXStartElement1(name, attrs);
    }
    void __SAXEndElement1(NetLoader *netLoader, const char *name){
        netLoader->SAXEndElement1(name);
    }
    void __SAXStartElement2(NetLoader *netLoader, const char *name, const char **attrs){
        netLoader->SAXStartElement2(name, attrs);
    }
    void __SAXEndElement2(NetLoader *netLoader, const char *name){
        netLoader->SAXEndElement2(name);
    }
    void __SAXError(NetLoader *netLoader, const char *msg, ...){
        cerr << msg << endl;
        netLoader->SAXError();
    }
} // extern "C"

NetLoader::NetLoader(){
}

NetLoader::~NetLoader(){
}



void NetLoader::SAXError(){
    saxErrors++;
}

void NetLoader::SAXStartElement1(string name, const char **attrs)
{
    if(name == "synapse"){
        return;
    } else if(name == "property"){
        if(attrs == NULL) return;
        ParseProperties(attrs);
        return;
    } else if(name == "neuron"){
        if(attrs == NULL) return;
        ParseNeuron(attrs);
        return;
    } else if(name == "topology"){
        if(attrs == NULL) return;
        ParseTopology(attrs);
        return;
    } else if(name == "network"){
        return;
    }
}


void NetLoader::SAXEndElement1(string name)
{
    if(name == "synapse"){
        sProps = NULL;
    }
    else if(name == "neuron"){
        NFactory * nf =  dynamic_cast<NFactory*> (FactoryBase::GetRegistry().GetFactory(currNType));
        if (nf == NULL) throw runtime_error("No neuron type " + currNType + " registered!");
        nf->MakeNeuron(currNId, nProps, currTop);
        currNId = 0;
        nProps = NULL;
    }
    else if(name == "topology"){
        currTop = NULL;
    }
}


void NetLoader::SAXEndElement2(string name)
{
    if(name == "neuron") {
        currNId = 0;
        currNType.clear();
        nProps = NULL;
    }else if(name == "synapse"){
        SpikingNeuron * post = dynamic_cast <SpikingNeuron * > (Network::GetNetworkRef()->GetNeuron(postNId));
        connector->Connect(currNeuron, post, *sProps);
        sProps = NULL;
    } else if(name == "outputgroup"){
        outputGroup = 0;
    } else if(name == "spikeinput"){
        spikeInput = NULL;
    } else if(name == "spikeoutput"){
        spikeOutput = NULL;
    }
}

void NetLoader::SAXStartElement2(string name, const char **attrs)
{

    if(name == "synapse"){
        if(attrs == NULL) return;
        ParseSynapse(attrs);
        return;
    } else if(name == "property"){
        if(attrs == NULL) return;
        ParseProperties(attrs);
        return;
    } else if(name == "neuron"){
        if(attrs == NULL) return;
        ParseNeuron(attrs);
        return;
    } else if(name == "topology"){
        if(attrs == NULL) return;
        ParseTopology(attrs);
        return;
    } else if(name == "outputgroup"){
        if(attrs == NULL) return;
        ParseOutputGroup(attrs);
        return;
    } else if(name == "spikeoutput"){
        if(attrs == NULL) return;
        ParseSpikeOutput(attrs);
        return;
    } else if(name == "spikeinput"){
        if(attrs == NULL) return;
        ParseSpikeInput(attrs);
        return;
    }
}

void NetLoader::ParseSpikeInput(const char **attrs)
{
    string siName;
    string siType;
    for (unsigned int i = 0; attrs[i] != NULL; i++) {
        string name(attrs[i]);
        string value(attrs[++i]);
        if ( name == "type" ) {
            siType = value;
        } else if ( name == "name" ) {
            siName = value;
        }
    }
    SIFactory * sif =  dynamic_cast<SIFactory*> (FactoryBase::GetRegistry().GetFactory(siType));
    if (sif == NULL) throw runtime_error("No SpikeInput type " + siType + " registered!");
    spikeInput = sif->MakeSpikeInput(siName);
}

void NetLoader::ParseSpikeOutput(const char **attrs)
{
    string soName;
    string soType;
    for (unsigned int i = 0; attrs[i] != NULL; i++) {
        string name(attrs[i]);
        string value(attrs[++i]);
        if ( name == "type" ) {
            soType = value;
        } else if ( name == "name" ) {
            soName = value;
        }
    }
    SOFactory * sof =  dynamic_cast<SOFactory*> (FactoryBase::GetRegistry().GetFactory(soType));
    if (sof == NULL) throw runtime_error("No SpikeOutput type " + soType + " registered!");
    spikeOutput = sof->MakeSpikeOutput(soName);
}


void NetLoader::ParseOutputGroup(const char **attrs)
{
    unsigned int enabled=0;
    for (unsigned int i = 0; attrs[i] != NULL; i++) {
        string name(attrs[i]);
        string value(attrs[++i]);
        if ( name == "id" ) {
            outputGroup = atoi(value.c_str());
        } else if ( name == "enabled" ) {
            enabled = atoi(value.c_str());
        }
    }
    OutputManager::EnableOutput(enabled, outputGroup);
}

void NetLoader::ParseSynapse(const char **attrs)
{
    string sType;
    AmTimeInt delay=0;
    for (unsigned int i = 0; attrs[i] != NULL; i++) {
        string name(attrs[i]);
        string value(attrs[++i]);
        if ( name == "postneuron" ){
            postNId = atoi(value.c_str());
        } else if ( name == "type" ) {
            sType = value;
        } else if ( name == "delay" ) {
            delay = atoi(value.c_str());
        }
    }
    ConnectorRegistry & cr = ConnectorRegistry::GetRegistry();
    connector = cr.GetConnector(sType);
    sProps = connector->GetDefaultProperties();
    sProps->SetDelay(delay);
}


void NetLoader::ParseNeuron(const char **attrs)
{
    for (unsigned int i = 0; attrs[i] != NULL; i++) {
        string name(attrs[i]);
        string value(attrs[++i]);
        if ( name == "id" ){
            currNId = atoi(value.c_str());
        } else if ( name == "type" ) {
            currNType = value;
        }
    }
    if(pass == 1) { // here we create only the neuronProperties. Neuron is created at end tag
        NFactory * nf =  dynamic_cast<NFactory*> (FactoryBase::GetRegistry().GetFactory(currNType));
        if (nf == NULL) throw runtime_error("No neuron type " + currNType + " registered!");
        nProps = nf->MakeNeuronProperties(true);
    } else if (pass == 2){ // get the neuron reference back to allow creating synapses and adding I/O
        currNeuron = Network::GetNetworkRef()->GetNeuron(currNId);
        if(spikeInput){
            InputNeuron * in = dynamic_cast <InputNeuron*> (currNeuron);
            if (in) spikeInput->AddNeuron(in);
            else cerr << "Warning: Cannot add neuron " << currNeuron->GetId()
                      << " because it's not an InputNeuron" << endl;
        } else if (outputGroup){
            OutputManager::AddNeuronToGroup(currNeuron, outputGroup);
        }
    }
}

void NetLoader::ParseProperties(const char **attrs)
{
   string key(attrs[1]);
   string value(attrs[3]);
   if(pass == 1 && nProps && !sProps){ // Neuron Properties
       nProps->SetProperty(key, value);
   } else if (pass == 2 && sProps){ // Synapse Properties
       sProps->SetProperty(key, value);
   }
}

void NetLoader::ParseTopology(const char **attrs)
{
    string topName;
    string topType;
    for (unsigned int i = 0; attrs[i] != NULL; i++) {
        string name(attrs[i]);
        string value(attrs[++i]);
        if ( name == "name" ){
            topName = value;
        } else if ( name == "type" ) {
            topType = value;
        }
    }

    if(pass == 1) {
        TFactory * tf =  dynamic_cast<TFactory*> (FactoryBase::GetRegistry().GetFactory(topType));
        if (tf == NULL) throw runtime_error("No topology type " + topType + " registered!");
        currTop = tf->MakeTopology(topName);
        Network::GetNetworkRef()->AddTopology(currTop);
    } else if (pass == 2){
        currTop = Network::GetNetworkRef()->GetTopology(topName);
    }
}



////////////////////////  End SAX callback stuff //////////////////////7



void NetLoader::LoadXML(string filename, bool _loadPhysProps){
    loadPhysProps = _loadPhysProps;
    LOGGER(2, "Starting SAX Pass 1 from file " + filename);
    pass = 1;
    SAXPass(filename);
    LOGGER(2, "Starting SAX Pass 2 from file " + filename);
    pass = 2;
    SAXPass(filename);
}


void NetLoader::WriteProperties(FILE * netFile, const Properties * props, const char * indent){
    try{
        map<string, string> propMap = props->GetPropertyMap();
        for(map<string, string>::const_iterator pPropIt = propMap.begin(); pPropIt != propMap.end(); pPropIt++){
            fprintf(netFile, "%s<property name=\"%s\" value=\"%s\"/>\n", indent,
                            pPropIt->first.c_str(), pPropIt->second.c_str());
        }
    } catch (runtime_error e) {
        cerr << "Warning: not saving Properties: ";
        cerr << e.what() << endl;
    }
}



void NetLoader::SaveXML(string filename, bool compress){

    string cmd;
    if (compress) {
	    cmd = "gzip -c > " + filename;
    }
    else {
	    cmd = "cat > " + filename;
    }

    FILE *netFile = popen(cmd.c_str(), "w");
    if(netFile == NULL) throw runtime_error("cannot open: " + filename);

    fputs ("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n", netFile);
    fputs ("<!DOCTYPE network PUBLIC \"amygdala.dtd\" \"dtd/amygdala.dtd\">\n\n", netFile);
    fputs ("<network>\n", netFile);

    Network * net = Network::GetNetworkRef();
    for (Network::const_iterator topIt = net->begin(); topIt != net->end(); topIt++){
        Topology * topology = topIt->second;
        fprintf(netFile, "    <topology name=\"%s\" type=\"%s\">\n",
                         topology->GetName().c_str(),
                         topology->GetClassName().c_str());

        nProps = topology->GetDefaultNeuronProperties();
        if(nProps != NULL) WriteProperties(netFile, nProps, "        ");
                         
        for(NeuronGroup::const_iterator nIt = topology->begin(); nIt != topology->end(); nIt++) {
            Neuron * n = *nIt;
            fprintf(netFile, "        <neuron type=\"%s\" id=\"%d\">\n",
                             n->GetClassName().c_str(), n->GetId());
            nProps = n->Properties(); // reuse pointer from above
            if(nProps != NULL) WriteProperties(netFile, nProps, "            ");
            Axon * axon = n->GetAxon();
            for (Axon::iterator aIt = axon->begin(); aIt != axon->end(); aIt++){
                AxonNode * aNode = *aIt;
                AxonNodeIterator anIt = aNode->Begin();
                Synapse *syn;
                while ((syn = anIt++)){
                    fprintf(netFile, "            <synapse type=\"%s\" postneuron=\"%d\" delay=\"%d\">\n",
                                     syn->GetClassName().c_str(),
                                     syn->GetDendrite()->GetPostNeuron()->GetId(),
                                     aNode->GetDelay());
                    sProps = syn->Properties();
                    // the delay is 0, we need to update this to the value obtained from the AxonNode
                    sProps->SetProperty("delay", Utilities::itostr(aNode->GetDelay()));
                    if(sProps != NULL) WriteProperties(netFile, sProps, "                ");
                    fputs("            </synapse>\n", netFile);
                }
            }
            fputs("        </neuron>\n", netFile);
        }
        fputs("    </topology>\n", netFile);

        for(Network::spikeinput_iterator siIt = net->SpikeInput_begin(); siIt != net->SpikeInput_end(); siIt++){
            SpikeInput *si = *siIt;
            fprintf(netFile, "    <spikeinput name=\"%s\" type=\"%s\">\n",
                             si->GetName().c_str(),
                             si->GetClassName().c_str());
            for(NeuronGroup::const_iterator nIt = si->begin(); nIt != si->end(); nIt++) {
                Neuron * n = *nIt;
                fprintf(netFile, "        <neuron id=\"%d\"/>\n", n->GetId());
            }
            fputs("    </spikeinput>\n", netFile);
        }
        fputs("    <output>\n", netFile);
        for(OutputManager::spikeoutput_iterator soIt = OutputManager::SpikeOutput_begin();
                soIt != OutputManager::SpikeOutput_end(); soIt++){
            SpikeOutput *so = soIt->second;
            fprintf(netFile, "        <spikeoutput name=\"%s\" type=\"%s\">\n",
                             so->GetName().c_str(),
                             so->GetClassName().c_str());
            for(NeuronGroup::const_iterator nIt = so->begin(); nIt != so->end(); nIt++) {
                Neuron * n = *nIt;
                fprintf(netFile, "            <neuron id=\"%d\"/>\n", n->GetId());
            }
            fputs("        </spikeoutput>\n", netFile);
        }
        for (OutputManager::output_group_iterator ogIt = OutputManager::OutputGroup_begin();
        		ogIt != OutputManager::OutputGroup_end(); ogIt++) {
	        AmGroupInt groupId = ogIt->first;
	        vector<Neuron*> nVec = ogIt->second;
	        fprintf(netFile, "        <outputgroup id=\"%d\" enabled=\"%d\">\n", groupId, Neuron::CaptureOutput(groupId));
	        
	        for (unsigned int j=0; j<nVec.size(); ++j) {
		        fprintf(netFile, "            <neuron id=\"%d\"/>\n", nVec[j]->GetId());
	        }
	        fputs("        </outputgroup>\n", netFile);
        }
        fputs("    </output>\n", netFile);
    }
    fputs("</network>\n", netFile);
    fclose(netFile);
}



void NetLoader::SAXPass(string filename){
    xmlSAXHandler saxHandler;
    memset (&saxHandler, 0, sizeof(saxHandler));
    

    currTop = NULL;
    currNId = 0;
    currNeuron = NULL;
    sProps = NULL;
    saxErrors = 0;
    spikeInput = NULL;
    spikeOutput = NULL;
    outputGroup = 0;
    
    xmlParserCtxtPtr ctxt = xmlCreateFileParserCtxt(filename.c_str());
    if (ctxt == NULL) throw runtime_error("can't create a parser context: " + filename);

    saxHandler.warning      = (warningSAXFunc)__SAXError;
    saxHandler.error        = (errorSAXFunc)__SAXError;
    saxHandler.fatalError   = (fatalErrorSAXFunc)__SAXError;
    switch(pass){
        case 1:
            saxHandler.startElement = (startElementSAXFunc)__SAXStartElement1;
            saxHandler.endElement   = (endElementSAXFunc)__SAXEndElement1;
            break;
        case 2:
            saxHandler.startElement = (startElementSAXFunc)__SAXStartElement2;
            saxHandler.endElement   = (endElementSAXFunc)__SAXEndElement2;
            break;
        default:
            throw runtime_error("Pass must be either 1 or 2");
            break;
    }

    ctxt->sax = &saxHandler;
    ctxt->userData = this;

    xmlParseDocument(ctxt);
    ctxt->sax = NULL;
    xmlFreeParserCtxt(ctxt);
    if(saxErrors > 0 )throw runtime_error ("Errors during loading...");
}

