/***************************************************************************
                          Network.cpp  -  description
                             -------------------
    copyright            : (C) 2001, 2004 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <amygdala/network.h>
#include <amygdala/spikingneuron.h>
#include <amygdala/inputneuron.h>
#include <amygdala/spikeinput.h>
#include <amygdala/netloader.h>
#include <amygdala/utilities.h>
#include <amygdala/amygdalaclass.h>
#include <amygdala/logging.h>
#include <amygdala/visualstub.h>
#include "amygdala/axon.h"
#include "amygdala/axonnode.h"
#include "amygdala/synapse.h"
#include "amygdala/functionlookup.h"
#include "amygdala/trainer.h"

#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <pthread.h>
#include <unistd.h>

#include <stdexcept>

#include <sys/stat.h>
#include <sys/types.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

using namespace std;
using namespace Amygdala;

// Callback function for the pthread library
/*extern "C" {
    void* NetworkThread(void* ){
        LOGGER(1, "Starting new thread");
        Network *Network = Network::GetNetworkRef();
        Network->Schedule();
        return new int(0);
    }
}*/

// to enable profiling threads on Linux.
// This code is shamelessly stolen from Samuel Hocevar. Thanks

#ifdef PROFILING
#include <sys/time.h>

/*extern "C" {
    typedef struct wrapper_s
    {
        void * (*start_routine)(void *);
        void * arg;

        pthread_mutex_t lock;
        pthread_cond_t  wait;

        struct itimerval itimer;

    } wrapper_t;

    static void * wrapper_routine(void *);

    int gprof_pthread_create(pthread_t * thread, pthread_attr_t * attr,
                             void * (*start_routine)(void *), void * arg)
    {
        wrapper_t wrapper_data;
        int i_return;

        wrapper_data.start_routine = start_routine;
        wrapper_data.arg = arg;
        getitimer(ITIMER_PROF, &wrapper_data.itimer);
        pthread_cond_init(&wrapper_data.wait, NULL);
        pthread_mutex_init(&wrapper_data.lock, NULL);
        pthread_mutex_lock(&wrapper_data.lock);

        i_return = pthread_create(thread, attr, &wrapper_routine,
                                                &wrapper_data);

        if(i_return == 0)
        {
            pthread_cond_wait(&wrapper_data.wait, &wrapper_data.lock);
        }

        pthread_mutex_unlock(&wrapper_data.lock);
        pthread_mutex_destroy(&wrapper_data.lock);
        pthread_cond_destroy(&wrapper_data.wait);

        return i_return;
    }

    static void * wrapper_routine(void * data)
    {
        void * (*start_routine)(void *) = ((wrapper_t*)data)->start_routine;
        void * arg = ((wrapper_t*)data)->arg;

        setitimer(ITIMER_PROF, &((wrapper_t*)data)->itimer, NULL);

        pthread_mutex_lock(&((wrapper_t*)data)->lock);
        pthread_cond_signal(&((wrapper_t*)data)->wait);
        pthread_mutex_unlock(&((wrapper_t*)data)->lock);

        return NetworkThread(arg);
    }
}*/
#endif //  PROFILING


//////////////////////////////////////////////////////////////////

Network * Network::theNetwork = NULL;
AmTimeInt Network::simTime = 0;
AmIdInt Network::nextNeuronId = 1;
AmTimeInt Network::simStepSize = 100;

Network::Network(int & argc, char * argv[]){
    srand(time(0));
    numThreads = 1;
    running = false;
    bool visualize = false;
    for(int i=0; i<argc; i++) {
        if(strstr(argv[i], "--visual")){
            visualize = true;
			// no stripping of the --visual argument here - Visual strips it
        }
    }
    if(visualize) {
        visualStub = new VisualStub();
        try {
            visualStub->Init(argc, argv);
        } catch (runtime_error e) {
            cerr << e.what() << endl;
            delete visualStub;
            visualStub = NULL;
        }
    }
    else {
	    visualStub=0;
    }
    
    funcLookup = new FunctionLookup();

    eventRequestCount = 0;
    maxSpikeDelay = 0;
    currSpikeDelayOffset = 0;
    maxOffset = 0;
    spikeBatchCount = 0;
}

void Network::Cleanup()
{
	delete theNetwork;
}

Network::~Network(){
    delete funcLookup;
}

void Network::ShiftArgs(int pos, int & argc, char *argv[]){
    if(pos == argc) return; // was the last arg
    for(int i=pos+1; i<argc; i++) {
        argv[i-1] = argv[i];
    }
    argc--;
}

void Network::Stop()
{
    maxRunTime = simTime;
}

static bool runInitialized = false;

void Network::Run(AmTimeInt _maxRunTime){
    running = true;
    maxRunTime = _maxRunTime;
    if (!runInitialized) {
        throw runtime_error("Network::InitRun() must be called before Network::Run()");
    }
    if(visualStub) {
        visualStub->RunVisual();
    }
    while(simTime < maxRunTime){
        // This will eventually be supplemented with multithreading code.
        TimeStep();
    }

    // NOTE:Threading code commented out until single-threaded mode is working again.
    // Much of this will have to change because partitions will not be used, but
    // some of it may be useful, so I have left it here.
    /*sleepers = 0;
    pthread_mutex_init(&mut_sleeper, NULL);
    pthread_mutex_init(&mut_simtime, NULL);
    pthread_cond_init(&cond_sleeper, NULL);

    if(numThreads == 1){     // no worker thread
        cout << "Running partitioned network, no threading" << endl;
        Schedule();          // to facilitate debugging
        running = false;
        return;
    }*/

    /*for(unsigned int threadCount = 0; threadCount < numThreads; threadCount++){
        pthread_t *theThread = new pthread_t;
        threadHandles.push_back(theThread);
#ifdef PROFILING
        gprof_pthread_create (theThread, NULL, NetworkThread, NULL);
#else
        pthread_create (theThread, NULL, NetworkThread, NULL);
#endif
        LOGGER(1, "Amygdala thread " << threadCount << " runnning ");
    }*/


    /*for(unsigned int threadCount = 0; threadCount < numThreads; threadCount++){
        int *retval;
        pthread_t *theThread = threadHandles[threadCount];
        pthread_join (*theThread, (void**)&retval);
        LOGGER(1, "Amygdala thread " << threadCount << " exited with status " << *retval)
        delete theThread;
        delete retval;
    }
    threadHandles.clear();*/
    running = false;
}

void Network::TimeStep(){
    simTime += TimeStepSize();
    LOGGER(4, "Network executing a timestep at time " << simTime)
    while (!eventQ.empty()) {

        const SpikeRequest& topSpike = eventQ.top();
        if (topSpike.spikeTime != simTime)
            break;

        topSpike.requestor->SendSpike(simTime);
        eventQ.pop();
    }

    // get the next input
    for (unsigned int i=0; i<spikeInputVec.size(); ++i) {
	    LOGGER(6, "Reading spike input buffer")
    	spikeInputVec[i]->ReadInputBuffer();
	}
	
    if (!inputQ.empty()) {
        nextInputTime = inputQ.top().spikeTime;
	    LOGGER(6, "Setting nextInputTime " << nextInputTime)
    }
    else {
        nextInputTime = 0;
    }

    while (nextInputTime <= simTime) {
        if (nextInputTime == 0)
            break;

        LOGGER(5, "Preparing to send input spike")
        inputQ.top().requestor->SendInputSpike(simTime);
        inputQ.pop();
        if (!inputQ.empty()) {
            nextInputTime = inputQ.top().spikeTime;
        }
        else {
            nextInputTime = 0;
        }
    }

    SendDelayedSpikes();

    // callbacks to any Trainers that are queued up
    if (!trainerCallbackQ.empty()) {
        AmTimeInt callTime = trainerCallbackQ.top().callbackTime;
        while (callTime <= simTime) {
            trainerCallbackQ.top().trainer->PeriodicTrain();
            trainerCallbackQ.pop();
            if (trainerCallbackQ.empty()) {
                break;
            }
            callTime = trainerCallbackQ.top().callbackTime;
        }
    }
}

void Network::InitRun(){
    runInitialized = true;
    
    // Initialize the topology objects
    map<std::string, Topology*>::iterator itr;
    for (itr=topologyMap.begin(); itr != topologyMap.end(); itr++) {
	    itr->second->InitRun();
    }

    // Initialize the delayed spike queue if it has not been done.
    if (!delayedSpikeQ.size()) {
        InitializeDelayedSpikeQ();
    }

    if (!inputQ.empty()) {
        nextInputTime = inputQ.top().spikeTime;
    }
    else {
        nextInputTime = 0;
    }
}

// FIXME: Commented out until I know what to do with it.  Has to be updated to match the
// new connection system.
/*void Network::ConnectNeurons(AmIdInt preInstId, AmIdInt preNId, AmIdInt postInstId, AmIdInt postNId, float weight, AmTimeInt delay){
    if(preInstId == postInstId) {
        Network *net = instances[preInstId]->GetNetwork();
        net->ConnectNeurons(preNId, postNId, weight, delay);
    } else {
        instances[preInstId]->GetNetwork()->ConnectNeurons(instances[postInstId], preNId, postNId, weight, delay);
    }
}*/


Network* Network::GetNetworkRef(){
    if (theNetwork == NULL) {
        string errstr = "No Network object defined. Call Network::Init(argc, argv) first";
        std::cerr << errstr << std::endl;
        throw runtime_error(errstr);
    }
    return theNetwork;
}


void Network::UdpListener(int port){
    // TODO: Implement this to enable clustering
}


AmTimeInt Network::GetMaxRunTime() { return maxRunTime; }

/*bool Network::ThreadSleep(unsigned int simTime){
    LOGGER(2, "Need to put threads to sleep. Maybe the partitioning is not optimal?");
    pthread_mutex_lock(&mut_sleeper);
    if(numThreads - sleepers == 1) {     // we are the last thread
        pthread_mutex_unlock(&mut_sleeper);
        return false;
    } else {
        sleepers++;
        pthread_cond_wait (&cond_sleeper, &mut_sleeper);
        pthread_mutex_unlock(&mut_sleeper);
        return true;
    }
}*/

/*void Network::ThreadWakeUp(){
    pthread_mutex_lock(&mut_sleeper);
    pthread_cond_broadcast (&cond_sleeper);
    sleepers = 0;
    pthread_mutex_unlock(&mut_sleeper);
}*/

void Network::Save(string filename, bool compress){
    if(running && numThreads > 1)
            throw runtime_error("Network::Save(): Cannot save, multithreaded Network is running");

    NetLoader nl;
    nl.SaveXML(filename, compress);
}


void Network::Load(string filename, bool loadPhysProps){
    if(running && numThreads > 1)
            throw runtime_error("Network::Load(): Cannot load, multithreaded Network is running");

    NetLoader nl;
    nl.LoadXML(filename, loadPhysProps);
}

// TODO: This will have to be changed with the new threading model.
// Comment it out for now until everything works in single-threaded mode
void Network::Schedule(){
/*    while(simTime < maxRunTime){
        bool didRun = false;
        for(map <AmIdInt, NetworkPartition*>::iterator part = partitions.begin(); part != partitions.end(); part++){
            NetworkPartition *partition = part->second;
            partition->RunLock();
            if(!partition->IsRunning() && (partition->SimTime() == simTime)) {
                partition->Running(true);
                partition->RunUnlock();
                partition->TimeStep();
                partition->Running(false);   // we should not need a lock to set running to false
                didRun = true;
            } else {
                partition->RunUnlock();
            }
        }
        if(!didRun) {  // couldn't find a Network to run. Going for the next timeStep
            for(map <AmIdInt, NetworkPartition*>::iterator part = partitions.begin(); part != partitions.end(); part++){
                NetworkPartition *partition = part->second;
                partition->RunLock();
                if(!partition->IsRunning() && (partition->SimTime() == simTime + Network::TimeStepSize())) {
                    partition->Running(true);
                    partition->RunUnlock();
                    partition->TimeStep();
                    partition->Running(false);   // we should not need a lock to set running to false
                    didRun = true;
                } else {
                    partition->RunUnlock();
                }
            }
        }
        if(!didRun) {  // even in the next timeStep there is no Network to run. Go to sleep.
            ThreadSleep(simTime);
        }

        bool incrementSimTime = true;
        pthread_mutex_lock(&mut_simtime);
        for(map <AmIdInt, NetworkPartition*>::iterator part = partitions.begin(); part != partitions.end(); part++){
            NetworkPartition *partition = part->second;
            if(partition->SimTime() == simTime) // only increment simTime when all
                incrementSimTime = false;   // Network::simTime are larger
        }
        if(incrementSimTime) {
            simTime += Network::TimeStepSize();
            LOGGER(3, "Incremented global Simtime to " << simTime)
            if(sleepers > 0)
                ThreadWakeUp();
        }
        pthread_mutex_unlock(&mut_simtime);
    }*/
}

void Network::SetNumThreads(unsigned int threads){
    if(running) throw runtime_error("Cannot change the number of threads while running");
    numThreads = threads;
}

void Network::SimpleRun(){
    //NetworkPartition *partition = partitions.begin()->second;
    if(visualStub) {
        visualStub->RunVisual();
    }
    cout << "Running single threaded" << endl;
    while(simTime < maxRunTime){
        TimeStep();
    }
}

void Network::Init(int & argc, char * argv[]){
    if (theNetwork != NULL) throw runtime_error("There is a Network already");
    theNetwork = new Network(argc, argv);
}

VisualStub * Network::GetVisualStub(){
    return visualStub;
}

Topology* Network::GetTopology(Neuron* nrn)
{
    return nrnMap[nrn->GetId()].topology;
}

void Network::AddNeuron(Neuron* nrn, Topology* top)
{
    if(nrnMap.find(nrn->GetId()) != nrnMap.end())
            throw runtime_error("Neuron " + Utilities::itostr(nrn->GetId()) + " exists already");
    NeuronTopologyPtr ntp;
    ntp.neuron = nrn;
    ntp.topology = top;
    nrnMap[nrn->GetId()] = ntp;
    if(nrn->GetId() >= nextNeuronId) nextNeuronId = nrn->GetId() + 1;
}

void Network::AddTrainer(const std::string& trainerName, Trainer* t)
{
    if (trainerMap.find(trainerName) != trainerMap.end())
        throw runtime_error("Trainer " + trainerName + " exists already");
    
    trainerMap[trainerName] = t;
}

void Network::AddTopology(Topology* top)
{
    topologyMap[top->GetName()] = top;
}

void Network::AddSpikeInput(SpikeInput* si)
{
	spikeInputVec.push_back(si);
}

/**********************************************************
 * Stuff from NetworkPartition.cpp
 **********************************************************/

void Network::ScheduleSpike(AmTimeInt spikeTime, SpikingNeuron* nrn)
{
    SpikeRequest newSpike;

    // initialize a new event request
    newSpike.requestTime = simTime;
    newSpike.requestor = nrn;
    newSpike.spikeTime = spikeTime;
    newSpike.requestOrder = eventRequestCount++;

    // insert into the queue
    eventQ.push(newSpike);
}

void Network::ScheduleSpike(AmTimeInt spikeTime, InputNeuron* nrn)
{
    InputSpikeRequest newSpike;

    // initialize a new event request
    newSpike.requestTime = simTime;
    newSpike.requestor = nrn;
    newSpike.spikeTime = spikeTime;
    newSpike.requestOrder = eventRequestCount++;

    // insert into the queue
    inputQ.push(newSpike);
}

void Network::ScheduleSpikeDelay(Axon* axon)
{
    LOGGER(6, "ScheduleSpikeDelay")
    for (Axon::iterator it= axon->begin(); it != axon->end(); it++) {
        LOGGER(6, "AxonNode size: " << (*it)->Size())
        AmTimeInt offset = (*it)->GetOffset();
        LOGGER(6, "AxonNode offset: " << offset)
        offset += currSpikeDelayOffset;

        // If the offset goes past the end of the queue, then
        // start back at the beginning
        if (offset > maxOffset) {
            offset -= (maxOffset + 1);
	        LOGGER(6, "Resetting offset to " << offset)
        }

        LOGGER(6, "Adding AxonNode to delayedSpikeQ")
        delayedSpikeQ[offset].push_back(*it);
    }
    spikeBatchCount += axon->size();
    LOGGER(6, "spikeBatchCount: " << spikeBatchCount)
}

void Network::SendDelayedSpikes()
{
	LOGGER(6, "currSpikeDelayOffset " << currSpikeDelayOffset)
    vector<AxonNode*>& nodes = delayedSpikeQ[currSpikeDelayOffset];
    if (!nodes.size()) {
        // Nothing to do
        IncrementDelayOffset();
        return;
    }

    LOGGER(6, "Parsing " << nodes.size() << " axon nodes")
    // parse each AxonNode and add each Synapse to a neuron
    // input processing queue.  neurons will call ScheduleNeuronProcess()
    // to trigger a call to Neuron::InputSpike() after the queueing process
    // is complete.
    for (unsigned int i=0; i<nodes.size(); ++i) {
        //AxonNode::const_iterator synItr = nodes[i]->begin();
	    //LOGGER(6, "Transmitting spikes from neuron " << nodes[i]->GetNeuron()->GetId())
	    LOGGER(6, "Getting AxonNodeIterator")
        AxonNodeIterator synItr = nodes[i]->Begin();
 
        Synapse* syn = 0;
        while ((syn = synItr++)) {
	        LOGGER(6, "Transmitting spike via synapse " << syn)
           	syn->TransmitSpike(simTime);
        }
    }

    // Allow each Neuron with input queued up to do
    // its processing
    LOGGER(6, processQ.size() << " items in processQ")
    for (unsigned int i=0; i<processQ.size(); ++i) {
        LOGGER(6, "Processing nrn " << processQ[i]->GetId())
        processQ[i]->ProcessInput(simTime);
    }
    processQ.clear();
    nodes.clear();
    IncrementDelayOffset();
}

void Network::InitializeDelayedSpikeQ()
{
	LOGGER(6, "Initializing delayedSpikeQ")
    maxOffset = maxSpikeDelay/TimeStepSize();
    LOGGER(6, "maxOffset: " << maxOffset)
    //delayedSpikeQ.reserve(maxOffset+1);
    //delayedSpikeQ.reserve(30);
    //LOGGER(6, "Space reserved for delayedSpikeQ")
    delayedSpikeQ.resize(maxOffset+1);
    //delayedSpikeQ.resize(30);
    LOGGER(6, "delayedSpikeQ resized")

    LOGGER(6, "Initializing delayedSpikeQ elements")
    for (unsigned int i=0; i<maxOffset+1; ++i) {
        // TODO: Assuming 100 delayed spikes per offset for now.
        // A more intelligent algorithm to determine proper
        // sizing should be developed later on.
        //vector<AxonNode*> tmpVec;
        //tmpVec.reserve(100);
        //delayedSpikeQ[i].push_back(tmpVec);
        delayedSpikeQ[i].reserve(100);
    }
}

void Network::SetMaxSpikeDelay(AmTimeInt _delay)
{
    if (_delay > maxSpikeDelay)
        maxSpikeDelay = _delay;
}

void Network::SetTrainerCallback(Trainer* t, AmTimeInt callbackTime)
{
    TrainerCallback tc;
    tc.trainer = t;
    tc.callbackTime = callbackTime;
    trainerCallbackQ.push(tc);
}
    
