#include "neuronproperties.h"
#include "physicalproperties.h"

using namespace Amygdala;
using namespace std;

NeuronProperties::NeuronProperties():Properties(),physProps(0)
{
}

NeuronProperties::NeuronProperties(bool initializePhysicalProps):
    Properties()
{
    if (initializePhysicalProps) {
        physProps = new PhysicalProperties();
    }
    else {
        physProps = 0;
    }
}

NeuronProperties::NeuronProperties(const NeuronProperties& rhs):
    Properties(rhs)
{   
    if (rhs.physProps) {
        physProps = new PhysicalProperties(*(rhs.physProps));
    }
    else {
        physProps = 0;
    }
}

NeuronProperties::~NeuronProperties(  )
{
    if(physProps)
        delete physProps;
}

void NeuronProperties::SetProperty(const string& name, const string& value)
{
    // No intrinsic properties, so just pass this on physProps
    if (physProps) {
        physProps->SetProperty(name, value);
    }
    else {
        // create a new PhysicalProperties and pass on
        // if the property is not found in PhysicalProperties,
        // memory leaks are not an issue because an exception will be
        // thrown which will alert the user
        physProps = new PhysicalProperties();
        physProps->SetProperty(name, value);
    }
}

map< string, string > NeuronProperties::GetPropertyMap() const
{
    if(physProps)
        return physProps->GetPropertyMap();
    else {
        map< string, string > empty;
        return empty;
    }
}
