/***************************************************************************
                          neuronproperties.h  -  description
                             -------------------
    copyright            : (C) 2003 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef NEURONPROPERTIES_H
#define NEURONPROPERTIES_H

#include <amygdala/properties.h>

namespace Amygdala {

class PhysicalProperties;

/** @class NeuronProperties neuronproperties.h amygdala/neuronproperties.h
  * @brief Properties used by all neurons
  * @see Properties
  * @author Matt Grover <mgrover@amygdala.org>
  */
class NeuronProperties: public Properties
{
public:
    NeuronProperties();
    /** @param initializePhysicalProps Flag to indicate whether or not
     * a PhysicalProperties object should be created. */
    NeuronProperties(bool initializePhysicalProps);
    /** Copy constructor */
    NeuronProperties(const NeuronProperties& rhs);
    virtual ~NeuronProperties() = 0;

    /** @return Ptr to PhysicalProperties obj */
    PhysicalProperties* GetPhysicalProps() const { return physProps; }
    /** Set the PhysicalProperties ptr */
    void SetPhysicalProps(PhysicalProperties* props) { physProps = props; }

    /** Make a copy of this NeuronProperties object
     * @return Ptr to a new NeuronProperties */
    virtual NeuronProperties* Copy() const = 0;

    /** @see Properties::SetProperty() */
    virtual void SetProperty(const std::string& name, const std::string& value);

    /** @see Properties::GetPropertyMap() */
    virtual std::map< std::string, std::string > GetPropertyMap() const;
protected:
    PhysicalProperties* physProps;
};
} // namespace Amygdala
#endif // NEURONPROPERTIES_H
