/***************************************************************************
                          simplenet.h  -  description
                             -------------------
    begin                : Fri Nov 21 2003
    copyright            : (C) 2003 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <amygdala/simplenet.h>
#include <amygdala/netloader.h>
#include <amygdala/genome.h>
#include <amygdala/neuron.h>
#include <amygdala/network.h>
#include <amygdala/logging.h>
#include <amygdala/physicalproperties.h>
#include <amygdala/neuronproperties.h>
#include <amygdala/basicneuron.h>
#include <amygdala/flatlayer.h>
#include <amygdala/nconnector.h>
#include <amygdala/synapse.h>
#include <amygdala/axon.h>
#include <amygdala/axonnode.h>
#include <amygdala/inputneuron.h>
#include <amygdala/utilities.h>

#include <vector>
#include <map>
#include <string>
#include <cmath>

#define CDIAG(X,Y,Z) sqrt((float(X))*(float(X)) + (float(Y))*(float(Y)) + (float(Z))*(float(Z)))


using namespace Amygdala;
using namespace std;

enum neurontype {INPUTNEURON, OUTPUTNEURON, HIDDENNEURON };


bool SimpleNet::cube_iterator::end = true;

void SimpleNet::GrowNeuron::CalculateType(const std::vector <Emitter> & sensitivities)
{
    float threshold = sensitivityThreshold;
    axonType = 0;
    vector <float> strengths(8);
    for (int i=0; i<8; i++) strengths[i] = 0.;
    for (vector<Emitter>::const_iterator it=sensitivities.begin(); it != sensitivities.end(); it++) {
        Emitter emitter = *it;
        if(emitter.type > 7) throw runtime_error("emitter type larger than 7 is not allowed");
        PhysicalProperties * pProps = neuron->Properties()->GetPhysicalProps();
        float distance = CDIAG(emitter.x - pProps->GetX(),
                               emitter.y - pProps->GetY(), emitter.z - pProps->GetZ());
        strengths[emitter.type] = emitter.strength / (distance*distance);
    }

    for (unsigned char type=0; type<8; type++){
        if(strengths[type] >= threshold) {
            axonType = type;
            threshold = strengths[type];
        }
    }
    LOGGER(5, "Neuron " << neuron->GetId() << " has type " << (unsigned int) axonType);
}


bool SimpleNet::GrowNeuron::GrowAxon(const std::vector <Emitter> & attractors)
{
    if(!growing) return growing;
    float strength1 = 0, strength2 = 0;
    float growvector[] = {0.,0.,0.};
    bool haveType = false;

    LOGGER(6, "Growing axon id = " << neuron->GetId() << " type " << (unsigned int)axonType);
    for( std::vector <Emitter>::const_iterator attrIt = attractors.begin(); attrIt != attractors.end(); attrIt++){
        Emitter attractor = *attrIt;
        if(attractor.type == axonType){
            haveType = true;
            // add to the grow vector
            float distance = CDIAG(attractor.x - axonTip.x , attractor.y - axonTip.y, attractor.z - axonTip.z);
            growvector[0] += attractor.strength * (attractor.x - axonTip.x) / distance;
            growvector[1] += attractor.strength * (attractor.y - axonTip.y) / distance;
            growvector[2] += attractor.strength * (attractor.z - axonTip.z) / distance;
            strength1 += attractor.strength / distance;
        }
    }
    if(!haveType){
        growing = false;
        return growing;
    }
    // scale the growth vector to a length of 0.1 and move the axontip
    float len = CDIAG(growvector[0], growvector[1], growvector[2]);
    for (unsigned int i = 0; i < 3; i++){
        growvector[i] *= (0.1 / len);
    }
    axonTip.x += growvector[0];
    axonTip.y += growvector[1];
    axonTip.z += growvector[2];
    axon.push_back(axonTip);

    // now figure out if growth finished
    for( std::vector <Emitter>::const_iterator attrIt = attractors.begin(); attrIt != attractors.end(); attrIt++){
        Emitter attractor = *attrIt;
        if(attractor.type == axonType){
            float distance = CDIAG(attractor.x - axonTip.x , attractor.y - axonTip.y, attractor.z - axonTip.z);
            strength2 += attractor.strength / distance;
        }
    }
    if (strength1 >= strength2)growing = false;
    TryMkSynapse();
    if(dynamic_cast <InputNeuron*>(neuron) == NULL){
        TryMkOutputSynapses();
    }
    return growing;
}


void SimpleNet::GrowNeuron::TryMkOutputSynapses()
{
    for(neuron_iterator it = simplenet->outputNeurons.begin(); it != simplenet->outputNeurons.end(); it++){
        GrowNeuron *postGrowNeuron = *it;
        Neuron *postNeuron = postGrowNeuron->neuron;
        if(AlreadyConnected(postNeuron)) continue;
        PhysicalProperties * pProps = postNeuron->Properties()->GetPhysicalProps();
        float distance = CDIAG(pProps->GetX() - axonTip.x ,
                               pProps->GetY() - axonTip.y,
                               pProps->GetZ() - axonTip.z);
        if(distance < 2.0) {
            SynLoc synloc = { { axonTip.x, axonTip.y, axonTip.z}, postNeuron };
            synLocs.push_back(synloc);
            LOGGER(4, "Adding synapse " << neuron->GetId() << " --> " << postNeuron->GetId());
            ConnectorRegistry & cr = ConnectorRegistry::GetRegistry();
//FIXME: StaticSynapse hardcoded
            NConnector * connector = cr.GetConnector("StaticSynapse");
            StaticSynapseProperties * sProps =
                    dynamic_cast <StaticSynapseProperties *> (connector->GetDefaultProperties());
//FIXME: valuse hardcoded
            sProps->SetDelay(1000);
            sProps->SetWeight(0.5);
            connector->Connect(neuron, static_cast <SpikingNeuron*>(postNeuron), *sProps);
        }
    }
}


bool SimpleNet::GrowNeuron::AlreadyConnected(Neuron * postNeuron)
{
    LOGGER(5, "Checking for synapse "<< neuron->GetId() << " --> " << postNeuron->GetId());
    for (synloc_iterator sIt = synLocs.begin(); sIt != synLocs.end(); sIt++){
        if(postNeuron->GetId() == (*sIt).neuron->GetId()){
            LOGGER(5, "Already connected: "<< neuron->GetId() << " --> " << postNeuron->GetId());
            return true;
        }
    }
    return false;
}


void SimpleNet::GrowNeuron::TryMkSynapse()
{
    int index[3];
    index[0] = (int) (axonTip.x+radius + 0.5);
    index[1] = (int) (axonTip.y+radius + 0.5);
    index[2] = (int) (axonTip.z+radius + 0.5);
    for (int i=0; i<3; i++) {
        if(index[i] <0 || index [i] > 2*radius){
            cerr << "Warning: Index out of range: " << __FILE__ << ":" << __LINE__ << endl;
            return;
        }
    }
    GrowNeuron *postGrowNeuron = simplenet->cube[index[0]] [index[1]] [index[2]];
    if(postGrowNeuron == NULL) return;
    Neuron *postNeuron = postGrowNeuron->neuron;
    if(postNeuron == neuron) return; // no synapses to myself
    if(AlreadyConnected(postNeuron)) return;
    SynLoc synloc = { { axonTip.x, axonTip.y, axonTip.z}, postNeuron };
    synLocs.push_back(synloc);
    LOGGER(4, "Adding synapse " << neuron->GetId() << " Axonpos " << axonTip.x << "," << axonTip.y << "," << axonTip.z  << 
              " --> " << postNeuron->GetId() << " at " << postNeuron->Properties()->GetPhysicalProps()->GetX() <<
              "," << postNeuron->Properties()->GetPhysicalProps()->GetY() <<
              "," << postNeuron->Properties()->GetPhysicalProps()->GetZ());
    LOGGER(5, "Adding synapse at index " << index[0] << ", " << index[1] << ", " << index[2]);
    ConnectorRegistry & cr = ConnectorRegistry::GetRegistry();
//FIXME: StaticSynapse hardcoded
    NConnector * connector = cr.GetConnector("StaticSynapse");
    StaticSynapseProperties * sProps =
            dynamic_cast <StaticSynapseProperties *> (connector->GetDefaultProperties());
//FIXME:
    sProps->SetDelay(2000);
    sProps->SetWeight(0.5);
    connector->Connect(neuron, static_cast <SpikingNeuron*> (postNeuron), *sProps);
}


void SimpleNet::GrowNeuron::InitAxon(const std::vector <Emitter> & sensitivities, Neuron * n)
{
    neuron = n;
    CalculateType(sensitivities);
    PhysicalProperties * pProps = n->Properties()->GetPhysicalProps();
    axonTip.x = pProps->GetX();
    axonTip.y = pProps->GetY();
    axonTip.z = pProps->GetZ();
    axon.push_back(axonTip);
    if(axonType != 0){
        growing = true;
    } else {
        growing = false;
    }
}


SimpleNet::GrowNeuron::GrowNeuron(SimpleNet *sn, const unsigned int _radius, Network * _net) : sensitivityThreshold(0.0), radius(_radius), simplenet(sn), net(_net)
{}

SimpleNet::GrowNeuron::~GrowNeuron(){}



SimpleNet::SimpleNet(string nType, float _oR, float _iR) : FACTOR(180./127.)
{
    cube = NULL;
    maxId = 0;
    outerRadius = _oR;
    innerRadius = _iR;
    oR = (int)ceil(outerRadius); // make an integer cube around sphere
    chromosome = 0;
    net = Network::GetNetworkRef();
    TopologyFactory<FlatLayer> makeFlatLayer;
    top = makeFlatLayer("single layer");
    neuronFactory = dynamic_cast<NFactory*>
                        (FactoryBase::GetRegistry().GetFactory(nType));
}

SimpleNet::~SimpleNet(){
    std::cout << "Destroying SimpleNet object" << std::endl;
    for(neuron_iterator it=inputNeurons.begin(); it != inputNeurons.end(); it++){
        delete *it;
    }
    for(neuron_iterator it=outputNeurons.begin(); it != outputNeurons.end(); it++){
        delete *it;
    }
    if(cube)DestroyCube();
}

void SimpleNet::DestroyCube()
{
    LOGGER(3, "Destroying the cube");
    for(int x=-oR; x<=oR; x++){
        for(int y=-oR; y<=oR; y++){
            for(int z=-oR; z<=oR; z++) {
                GrowNeuron *gn = cube[x+oR][y+oR][z+oR];
                if(gn) delete gn;
            }
            delete [] cube[x+oR][y+oR];
        }
        delete [] cube[x+oR];
    }
    delete [] cube;
}


void SimpleNet::AddInputNeuron(float phi, float psi, AmIdInt id){
    if(id > 255) throw runtime_error("IDs of input neurons must be <= 255");
    std::string gene;
    gene.push_back((signed char)(phi/FACTOR));
    gene.push_back((signed char)(psi/FACTOR));
    gene.push_back((unsigned char)id);
    inputChromosome.push_back(gene);
}


void SimpleNet::AddOutputNeuron(float phi, float psi, AmIdInt id){
    if(id > 255) throw runtime_error("IDs of output neurons must be <= 255");
    std::string gene;
    gene.push_back((signed char)(phi/FACTOR));
    gene.push_back((signed char)(psi/FACTOR));
    gene.push_back((unsigned char)id);
    outputChromosome.push_back(gene);
}


void SimpleNet::Upload(string uri){
    Genome g(uri);
    MakeIOChromosome(inputChromosome, g);
    MakeIOChromosome(outputChromosome, g);
    g.MakeChromosome(20, 5, 'v');     // Sensitivities
    g.MakeChromosome(20, 5, 'v');     // Attractors
    try {
        g.Upload();
    } catch (runtime_error e) {
        std::cout << e.what() << std::endl;
        throw e;
    }
}

void SimpleNet::FinishParsing()
{
    LOGGER(3, "Finished parsing the genome");
    MakeCube();
}


bool SimpleNet::Step()
{
    LOGGER(2, "Doing one step of axon growth");
    bool lastStep = true;

    for (cube_iterator it = begin(); it != end(); it++){
        GrowNeuron *growNeuron = *it;
        if(growNeuron->GrowAxon(attractors)){
            lastStep = false;
        }    
    }
    for(neuron_iterator it = input_begin(); it != input_end(); it++){
        GrowNeuron *growNeuron = *it;
        if(growNeuron->GrowAxon(attractors)){
            lastStep = false;
        }    
    }
    // Output neurons don't grow axons anyway so they're left out
    
    if (lastStep) {
        MakeInhibitories();
        OutputManager::EnableOutput(true, 1);
        LOGGER(3, "morphogenesis done");
        return false;
    }
    return true;
}

void SimpleNet::MakeEmitter(Emitter & emitter, const string & gene)
{
    const float factor = outerRadius / 127.;
    emitter.type = ((unsigned char)gene[0]) & 0x7; // only the last 3 bit
    emitter.strength = ((float)(unsigned char)gene[1]) / 256.;
    emitter.x = factor * (float)(signed char)gene[2];
    emitter.y = factor * (float)(signed char)gene[3];
    emitter.z = factor * (float)(signed char)gene[4];
    LOGGER(2, "Made an emitter of type " << (int)emitter.type << " strength " << emitter.strength);
}


void SimpleNet::MakeIONeuron(const std::string & gene, int type)
{
    float phi =  FACTOR * (float)(signed char)gene[0];
    float psi =  FACTOR * (float)(signed char)gene[1];
    float x = oR * sin(phi) * cos(psi);
    float y = oR * sin(phi) * sin(psi);
    float z = oR * cos(phi);
    AmIdInt id =  (AmIdInt)(unsigned char)gene[2];
    if(id>maxId) maxId = id;
    GrowNeuron *sNNeuron = new GrowNeuron(this, oR, net);

    if(type == INPUTNEURON){
        string nType = "InputNeuron";
        LOGGER(3, "Making InputNeuron " << id << " at location " << x << "  " << y << "  " << z);
        InputNeuronProperties iProps(1);
        iProps.GetPhysicalProps()->SetPosition(x, y, z);
        PhysicalProperties::RGB color = {0, 88, 188};
        iProps.GetPhysicalProps()->SetBodyColor(color);
        NFactory * iFactory = dynamic_cast<NFactory*>
                            (FactoryBase::GetRegistry().GetFactory(nType));
        if(iFactory == NULL) throw runtime_error("Cannot get a NeuronFactory for input");
        Neuron * n = iFactory->MakeNeuron(id, &iProps, top);
        if(n == NULL) throw runtime_error(string("Creating InputNeuron id ") + Utilities::itostr(id) + "failed");
        sNNeuron->InitAxon(sensitivities, n);
        inputNeurons.push_back(sNNeuron);
        OutputManager::AddNeuronToGroup(n, 2);
    } else {
        string nType = "BasicNeuron";
        LOGGER(3, "Making OutputNeuron " << id << " at location " << x << "  " << y << "  " << z);
        BasicNeuronProperties bProps(1);
        bProps.GetPhysicalProps()->SetPosition(x, y, z);
        PhysicalProperties::RGB color = {120, 0, 20};
        bProps.GetPhysicalProps()->SetBodyColor(color);
        NFactory * iFactory = dynamic_cast<NFactory*>
                            (FactoryBase::GetRegistry().GetFactory(nType));
        if(iFactory == NULL) throw runtime_error("Cannot get a NeuronFactory for output");
        Neuron * n = iFactory->MakeNeuron(id, &bProps, top);
        if(n == NULL) throw runtime_error(string("Creating OutputNeuron id ") + Utilities::itostr(id) + "failed");
        sNNeuron->InitAxon(sensitivities, n);
        sNNeuron->Growing(false);
        outputNeurons.push_back(sNNeuron);
        OutputManager::AddNeuronToGroup(n, 1);
    }
}

void SimpleNet::Gene(const string & gene, AmIdInt chromosomeId){
    Emitter emitter;
    switch (chromosomeId) {
        case 0:          // fixed chromosome for input neurons
            MakeIONeuron(gene, INPUTNEURON);
            break;
        case 1:          // fixed chromosome for output neurons
            MakeIONeuron(gene, OUTPUTNEURON);
            break;
        case 2:          // sensitivities
            MakeEmitter(emitter, gene);
            sensitivities.push_back(emitter);
            break;
        case 3:          // attractors
            MakeEmitter(emitter, gene);
            attractors.push_back(emitter);
            break;
        default:
            throw runtime_error("Unexpected chromosome");
            break;
    }
}

void SimpleNet::MakeCube()
{
    BasicNeuronProperties bProps(1);
    maxId += 10;       // make a gap which can be seen in the spikeplot
    LOGGER(1, "Making neurons, radius of sphere is " << oR << " within " << outerRadius << " hollow = " << innerRadius);
    cube = new GrowNeuron ***[2*oR+1];
    // iterate through a qube of length 2*outerRadius
    for(int x=-oR; x<=oR; x++){
        cube[x+oR] = new GrowNeuron**[2*oR+1];
        for(int y=-oR; y<=oR; y++){
            cube[x+oR][y+oR] = new GrowNeuron*[2*oR+1];
            for(int z=-oR; z<=oR; z++){
                float r = CDIAG(x, y, z);
                if(r<=outerRadius && r>=innerRadius){
                    bProps.GetPhysicalProps()->SetPosition(float(x), float(y), float(z));
                    GrowNeuron *sNNeuron = new GrowNeuron(this, oR, net);
                    Neuron * n =
                        dynamic_cast<Neuron*>
                        (neuronFactory->MakeNeuron(maxId++, &bProps, top));
                    sNNeuron->InitAxon(sensitivities, n);
                    cube[x+oR][y+oR][z+oR] = sNNeuron;
                    OutputManager::AddNeuronToGroup(n, 3);
                } else {
                    cube[x+oR][y+oR][z+oR] = NULL;
                }
            }
        }
    }
}

void SimpleNet::MakeIOChromosome (std::vector<std::string> & neurons, Amygdala::Genome & g) {
    g.StartChromosome('c');
    for(std::vector<std::string>::const_iterator it=neurons.begin(); it != neurons.end(); it++){
            g.AddGene(*it);
    }
}


std::vector < AmIdInt > SimpleNet::GetInputIDs()
{
    std::vector < AmIdInt > input;
    for (std::vector <GrowNeuron *>::const_iterator it = inputNeurons.begin(); it != inputNeurons.end(); it++){
        input.push_back((*it)->GetNeuron()->GetId());
    }
    return input;
}

std::vector < AmIdInt > SimpleNet::GetOutputIDs()
{
    std::vector < AmIdInt > output;
    for (std::vector <GrowNeuron *>::const_iterator it = outputNeurons.begin(); it != outputNeurons.end(); it++){
        output.push_back((*it)->GetNeuron()->GetId());
    }
    return output;
}


void SimpleNet::MakeInhibitories()
{
    ConnectorRegistry & cr = ConnectorRegistry::GetRegistry();
    NConnector * connector = cr.GetConnector("StaticSynapse");
    StaticSynapseProperties * sProps =
            dynamic_cast <StaticSynapseProperties *> (connector->GetDefaultProperties());
    
    sProps->SetDelay(1000);
    sProps->SetWeight(-0.5);
    for(int x=-oR; x<=oR; x++){
        for(int y=-oR; y<=oR; y++){
            for(int z=-oR; z<=oR; z++){
                GrowNeuron *gPre = cube[x+oR][y+oR][z+oR];
                if(gPre == NULL)continue;
                for(int xp = x-1; xp<=x+1; xp++){
                    if(xp < -oR || xp > oR) continue;
                    for(int yp = y-1; yp<=y+1; yp++){
                        if(yp < -oR || yp > oR) continue;
                        for(int zp = z-1; zp<=z+1; zp++){
                            if(zp < -oR || zp > oR) continue;
                            GrowNeuron * gPost = cube[xp+oR][yp+oR][zp+oR];
                            if(gPost == NULL || gPre == gPost)continue;
                            connector->Connect(gPre->GetNeuron(), static_cast <SpikingNeuron*>(gPost->GetNeuron()), *sProps);
                        }
                    }
                }
            }
        }
    }
}
