/***************************************************************************
                          spikingneuronproperties.h  -  description
                             -------------------
    copyright            : (C) 2003 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef SPIKINGNEURONPROPERTIES_H
#define SPIKINGNEURONPROPERTIES_H

#include <amygdala/neuronproperties.h>
#include <string>
#include <map>
#include <vector>

namespace Amygdala {

/**
  * @class SpikingNeuronProperties spikingneuronproperties.h amygdala/spikingneuronproperties.h
  * @brief Properties common to all SpikingNeurons
  * @see NeuronProperties, SpikingNeuron
  * @author Matt Grover <mgrover@amygdala.org> */
class SpikingNeuronProperties : public NeuronProperties
{
public:
    SpikingNeuronProperties();
    /** @param initializePhysicalProps Flag to indicate if PhysicalProperties object should
     * be created */
    SpikingNeuronProperties(bool initializePhysicalProps);
    /** Copy constructor */
    SpikingNeuronProperties(const SpikingNeuronProperties& rhs);
    virtual ~SpikingNeuronProperties();

    /** @return The absolute refractory period (microseconds) */
    AmTimeInt GetRefractoryPeriod() const { return refPeriod; }
    
    /** Set the absolute refractory period 
     * @param period Refractory period in microseconds */
    void SetRefractoryPeriod(AmTimeInt period) { refPeriod = period; }

    virtual void SetProperty(std::string& name, std::string& value);

    virtual SpikingNeuronProperties* Copy() const = 0;

    virtual std::map< std::string, std::string > GetPropertyMap() const;
protected:
    AmTimeInt refPeriod;                // Refractory time period (us)
private:
};
} // namespace Amygdala
#endif // SPIKINGNEURONPROPERTIES_H
