/***************************************************************************
                          synapseproperties.h  -  description
                             -------------------
    copyright            : (C) 2004 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef SYNAPSEPROPERTIES_H
#define SYNAPSEPROPERTIES_H


#include <amygdala/properties.h>

namespace Amygdala {
/**
  * @class SynapseProperties synapseproperties.h amygdala/synapseproperties.h
  * Properties needed to set up Synapses.
  * @see Synapse, NeuronConnector
  * @author Matt Grover <mgrover@amygdala.org> 
  */
class SynapseProperties: public Properties
{
/** Public methods: */
public:
    SynapseProperties(AmTimeInt _delay);
    virtual ~SynapseProperties() = 0;

    /** @return Synaptic delay (microseconds) */
    AmTimeInt GetDelay() const { return delay; }
    /** Set the synaptic delay (microseconds) */
    void SetDelay(AmTimeInt _delay) { delay = _delay; }
protected:
    AmTimeInt delay;
};

/** @class StaticSynapseProperties synapseproperties.h amygdala/synapseproperties.h
  * @see StaticSynapse 
  * @author Matt Grover <mgrover@amygdala.org> 
  */
class StaticSynapseProperties: public SynapseProperties {
public:
    StaticSynapseProperties(AmTimeInt _delay):SynapseProperties(_delay) {};
    StaticSynapseProperties(float _weight, AmTimeInt _delay);
    virtual ~StaticSynapseProperties();

    /** @return A static weight value */
    float GetWeight() const { return weight; }
    /** Set the static weight */
    void SetWeight(float _weight) { weight = _weight; }

    virtual void SetProperty(const std::string& name, const std::string& value);

    virtual std::map< std::string, std::string > GetPropertyMap() const;
protected:
    float weight;
};

/** @class DynamicSynapseProperties synapseproperties.h amygdala/synapseproperties.h
 * @see DynamicSynapse
 * @author Rudiger Koch <rkoch@amygdala.org> */
 
class DynamicSynapseProperties: public SynapseProperties {
public:
    DynamicSynapseProperties(AmTimeInt _delay);
    virtual ~DynamicSynapseProperties();
    
    /** Set one dynamic parameter of C, U, D */
    void SetDynaParam(const char param, float value);
    /** Get one dynamic parameter of C, U, D */
    const float & GetDynaParam(const char param);

    virtual void SetProperty(const std::string& name, const std::string& value);

    virtual std::map< std::string, std::string > GetPropertyMap() const;
protected:
    float A, U, D, F; // dynamic parameters
};
} // namespace Amygdala
#endif // SYNAPSEPROPERTIES_H

