/***************************************************************************
                          chromosome.cpp  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "chromosome.h"
#include "genome.h"
#include "gene.h"
#include "amygdala/logging.h"


Chromosome::Chromosome(){
}
Chromosome::~Chromosome(){
    LOGGER(5, "deleting chromosome");
    for(unsigned int i=0; i<theChromosome.size(); i++) delete theChromosome[i];
}


void Chromosome::Generate(unsigned int genes, unsigned int _gene_size){
    gene_size = _gene_size;
    theChromosome.clear();
    for(unsigned int i=0; i<genes; i++){
        theChromosome.push_back(new Gene(gene_size));
    }
}


string Chromosome::ToString(){
    string chromosome;
    for(unsigned int i=0; i<theChromosome.size(); i++){
        chromosome += theChromosome[i]->GetGene();
    }
    return chromosome;
}


Chromosome::Chromosome(unsigned int genes, unsigned int _gene_size, unsigned char _mutationMode, string chromosomeString){
    LOGGER (6, "Copy chromosome, " << genes << " genese of size " << _gene_size << ". Total length: " << chromosomeString.length());
    gene_size = _gene_size;
    mutationMode = _mutationMode;
    for(unsigned int i=0; i<genes; i++){
        theChromosome.push_back(new Gene(chromosomeString.substr(i * (gene_size+MUTATION_HEAT), gene_size+MUTATION_HEAT)));
    }
}


void Chromosome::Crossover(Chromosome *male){
    unsigned int i;
    float lc = atof(Genome::GetAttribute("likely_crossover").c_str());
    for(i=0; i<theChromosome.size(); i++){
        if(male->GetNumGenes() > i){
            if((100.*rand()/RAND_MAX) < lc) {
                theChromosome[i]->SetGene(male->theChromosome[i]->GetGene());
            }
        }
    }
    // if the male genome is longer...
    for(; i<male->GetNumGenes(); i++){
        if((100.*rand()/RAND_MAX) < lc) {
            LOGGER(6, "Male chromosome is longer, appending a gene");
            theChromosome.push_back(new Gene(male->theChromosome[i]->GetGene()));
        }
    }
}

void Chromosome::Mutate(){
    if(mutationMode == 'c') return;
    float lmf = atof(Genome::GetAttribute("likely_mutation_flip").c_str());
    float lmPM = atof(Genome::GetAttribute("likely_mutation_pm").c_str());
    float lmDup = atof(Genome::GetAttribute("likely_mutation_duplication").c_str());
    float lmDel = atof(Genome::GetAttribute("likely_mutation_deletion").c_str());
    
    const float lmMutationHeat = 5.;
    bool mutationHeatOn = false;
    
    if(Genome::GetAttribute("mutation_heat") == "on"){
        LOGGER(5, "Going to honor mutation heat");
        mutationHeatOn = true;
    }

    for(unsigned int i=0; i<theChromosome.size(); i++){
        LOGGER(6, "Mutating gene " << i);
        float mutationHeat;
        if(mutationHeatOn){
            mutationHeat = theChromosome[i]->GetMutationHeat();
            if((100.*rand()/RAND_MAX) < lmMutationHeat) { // Not subject of mutation heat
                theChromosome[i]->MutateHeat();
            }
        } else {
            mutationHeat = 1.0;
        }
        
        if((100.*rand()/RAND_MAX) < (lmPM * mutationHeat)) {
            theChromosome[i]->MutatePM();
        }
        if((100.*rand()/RAND_MAX) < (lmf * mutationHeat)) {
            theChromosome[i]->MutateFlip();
        }
        if(mutationMode == 'v'){
            if((100.*rand()/RAND_MAX) < (lmDup * mutationHeat)) {
                LOGGER(6, "Copying gene " << i << "to the end of the chromosome");
                // this copies the current gene to the end of the chromosome
                theChromosome.push_back(new Gene(theChromosome[i]->GetGene()));
            }
        }
    }
    if(mutationMode == 'v'){
        // The last gene doesn't have a protection from deletion by mutation_heat
        if((100.*rand()/RAND_MAX) < lmDel) {
            LOGGER(6, "Deleting last gene");
            theChromosome.pop_back();
        }
    }
}

const unsigned int Chromosome::GetNumGenes() const
{
    return theChromosome.size();
}


const unsigned int Chromosome::GetGeneSize() const
{
    return gene_size;
}


const unsigned char Chromosome::GetMutationMode() const
{
    return mutationMode;
}
