/***************************************************************************
                          chromosome.h  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHROMOSOME_H
#define CHROMOSOME_H

#include <vector>
#include <string>


class Gene;
using namespace std;

/**A string of genes with the same format
  *@author Rudiger Koch
  */

class Chromosome {
public: 
    Chromosome();
    ~Chromosome();

    /** generate all the genes for this chromosome */
    void Generate(unsigned int genes, unsigned int _gene_size);
    /** concatenate all genes and return it */
    string ToString();
    /** construct a chromosome from a string */
     Chromosome(unsigned int genes, unsigned int gene_size, unsigned char mutationMode, string chromosomeString);
    /** mutation operator */
    void Mutate();
    /** Crossover operator */
    void Crossover(Chromosome *male);
    /** @return the number of genes of this chromosome */
    const unsigned int GetNumGenes() const;
    /** @return the size of the genes of this chromosome */
    const unsigned int GetGeneSize() const;
    const unsigned char GetMutationMode() const;

protected: // Protected attributes
    /**  The genes of this chomosome */
    vector <Gene*> theChromosome;

    /** the REAL gene size, that is: including the heat byte */    
    unsigned int gene_size;
    /** 'n'= non mutable; 'c' = constant size; 'v'= variable size */
    char mutationMode;
};

#endif
