/***************************************************************************
                          gfile.cpp  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gfile.h"
#include "genome.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <stdio.h>

#include <algorithm>
#include <stdexcept>
#include <fstream>
#include <iostream>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# if HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# if HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# if HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif


pthread_mutex_t GFile::mut_file;


GFile::GFile(){
}

GFile::GFile(int _id){
    id = _id;
}
GFile::~GFile(){
}

void GFile::Lock() {
    pthread_mutex_lock(&mut_file);
}
void GFile::Unlock() {
    pthread_mutex_unlock(&mut_file);
}
void GFile::InitLock(){
    pthread_mutex_init(&mut_file, NULL);
}



void GFile::Checkin(int score){
    char fileName[14];
    sprintf(fileName, "%08d.gene", id);
    string outFileName = Genome::GetAttribute("geneserver_dir") + string("/out/") + string(fileName);
    string inFileName = Genome::GetAttribute("geneserver_dir") + string("/incoming/") + string(fileName);
    if(rename(outFileName.c_str(), inFileName.c_str()) == -1) throw runtime_error("No such gene ID");

    string fitnessfile = Genome::GetAttribute("geneserver_dir") + string("/incoming/fitness");
    FILE *fitnessfp = fopen(fitnessfile.c_str(), "a");
    if(!fitnessfp) throw runtime_error("Server Error - cannot open fitness file");

    fprintf(fitnessfp, "%08d %d\n", id, score);
    fclose(fitnessfp);
}



string GFile::Checkout(){
    string fileName = GetNextGene(Genome::GetAttribute("geneserver_dir") + string("/outgoing"));
    if(fileName.empty()){
        if(CountIncoming() < atoi(Genome::GetAttribute("population").c_str())){
            CopyGene();
        } else {
            Genome::MateAll(SelectFitest());
        }
        fileName = GetNextGene(Genome::GetAttribute("geneserver_dir") + string("/outgoing"));
        if(fileName.empty())throw runtime_error(fileName + ": cannot open");
    }

    FILE *geneFile = fopen(fileName.c_str(), "r");
    if(!geneFile) throw runtime_error(fileName + ": cannot open");

    int c;
    string genome;
    while((c = getc(geneFile)) != EOF) genome.push_back(c);
    fclose(geneFile);

    string newFileName = Genome::GetAttribute("geneserver_dir") + string("/out") +
                         fileName.substr(fileName.find_last_of('/'));
    rename(fileName.c_str(), newFileName.c_str());

    return genome;
}


void GFile::Save(string genome){
    string outgoingdir = Genome::GetAttribute("geneserver_dir")  + "/outgoing";
    char *fileName = new char[outgoingdir.size() + 15];
    sprintf(fileName, "%s/%08d.gene", outgoingdir.c_str(), id);

    ofstream geneFile(fileName);
    if(!geneFile.good()) return;  // TODO: Exception
    geneFile << genome;
    geneFile.close();

    delete [] fileName;
}



const int& GFile::GetId(){
	return id;
}


string GFile::Load(){
    string incomingdir = Genome::GetAttribute("geneserver_dir") + string("/incoming");
    char *fileName = new char[incomingdir.size() + 15];
    sprintf(fileName, "%s/%08d.gene", incomingdir.c_str(), id);

    ifstream geneFile(fileName);
    string genome;
    if(!geneFile.good()) return genome;  // TODO: Exception
    while(!geneFile.eof()) genome.push_back(geneFile.get());
    geneFile.close();

    delete [] fileName;
    return genome;
}



int * GFile::SelectFitest(){
    string incoming_dir = Genome::GetAttribute("geneserver_dir") + string("/incoming");
    string fileName = incoming_dir + string("/fitness");
    FILE *fitnessFile;
    if(!(fitnessFile = fopen(fileName.c_str(), "r"))){
        cerr << "Cannot open file: " << fileName << endl;
        exit(2);
    }
    vector <Individual> incoming;
    int _id, _score;
    while(fscanf(fitnessFile, "%d %d\n", &_id, &_score) != EOF){
        Individual individual;
        individual.id    = _id;
        individual.score = _score;
        incoming.push_back(individual);
    }
    fclose(fitnessFile);
    sort(incoming.begin(), incoming.end());

    unsigned int survivors = atoi(Genome::GetAttribute("survivors").c_str());
    int alltime_best = atoi(Genome::GetAttribute("alltime_best").c_str());
    int *fitest = new int[survivors+1];
    for(unsigned int i=0; i<incoming.size(); i++) {
        if(incoming[i].score >= alltime_best){   // save this individual
            char fileName[16];
            sprintf(fileName, "%08d.gene", incoming[i].id);
            string cmd = string("cp ") + Genome::GetAttribute("geneserver_dir") + string("/incoming/") +
                         string(fileName) + string(" ") + Genome::GetAttribute("geneserver_dir") + string("/best");
            Copy (Genome::GetAttribute("geneserver_dir") + "/incoming/" + fileName,
                  Genome::GetAttribute("geneserver_dir") + "/best/" + fileName);
        }
        if(i < survivors) {
            fitest[i] = incoming[i].id;
        } else {
            if(i == survivors) fitest[i] = -1;
            char *geneFile = new char[incoming_dir.size() + 15];
            sprintf(geneFile, "%s/%08d.gene", incoming_dir.c_str(), incoming[i].id);
            remove(geneFile);
            delete [] geneFile;
        }
    }
    return fitest;
}

void GFile::Copy(string srcfile, string dstfile)
{
    FILE * src = fopen(srcfile.c_str(), "r");
    FILE * dst = fopen(dstfile.c_str(), "w");
    if(src == NULL || dst ==NULL)
        throw runtime_error("Cannot copy " + srcfile + " to " + dstfile);
    int aByte;
    while((aByte = getc(src)) != EOF){
        putc(aByte, dst);
    }
    fclose(src);
    fclose(dst);
}

int GFile::CountIncoming(){
    char dummy[40];
    string fileName = Genome::GetAttribute("geneserver_dir") + string("/incoming") + string("/fitness");
    ifstream fitnessFile(fileName.c_str());
    if(!fitnessFile.good())return 0;
    int lines = 0;
    while(!fitnessFile.eof()){
        fitnessFile.getline(dummy, 40);
        lines++;
    }
    fitnessFile.close();
    return lines;
}



void GFile::CopyGene(){
    string fileName = GetNextGene(Genome::GetAttribute("geneserver_dir") + "/out");
    id = IncrementCounter();
    string outgoingdir = Genome::GetAttribute("geneserver_dir") + "/outgoing/";
    char newFileName[25];
    sprintf(newFileName, "%08d.gene", id);
    Copy(fileName, outgoingdir + newFileName);
}



string GFile::GetNextGene(string dir){
    struct dirent **namelist;
    int n;
    string a_gene;

    n = scandir(dir.c_str(), &namelist, 0, alphasort);
    if(n==-1) throw runtime_error ("Cannot scan directory");
    while(n--) {
        if(!strcmp(namelist[n]->d_name + strlen(namelist[n]->d_name) - 5, ".gene") && a_gene.empty())
            a_gene = namelist[n]->d_name;
        free(namelist[n]);
    }
    free(namelist);

    if(a_gene.empty()) return "";
    id = atoi(a_gene.c_str());
    return dir + "/" + a_gene;
}

int GFile::FitnessCompare(const void* _i1, const void* _i2){
    Individual* i1 = (Individual*) _i1;
    Individual* i2 = (Individual*) _i2;
    return i2->score - i1->score;
}



int GFile::IncrementCounter(){
    int maxId;
    string counterFileName = Genome::GetAttribute("geneserver_dir") + string("/counter");

    FILE *counterFile;
    if(!(counterFile = fopen(counterFileName.c_str(), "r+"))){
        cerr << "Cannot open counter file: " << counterFileName << endl;
        exit(5);
    }
    fscanf(counterFile, "%d", &maxId);
    rewind(counterFile);
    maxId++;
    fprintf(counterFile, "%d", maxId);
    fclose(counterFile);
    return maxId;
}


void GFile::Die(){
    string incomingdir = Genome::GetAttribute("geneserver_dir") + "/incoming/";
    char fileName[25];
    sprintf(fileName, "%08d.gene", id);
    remove((incomingdir + fileName).c_str());
}


void GFile::EmptyDir(string dir)
{
    struct dirent *entry;
    DIR * cdir = opendir(dir.c_str());
    while ((entry = readdir(cdir))){
       string geneFile = entry->d_name;
       if(geneFile != "." && geneFile != "..") { // skip links to directories
            if(unlink((dir + "/" + geneFile).c_str())){
                throw runtime_error("Cannot unlink: " + geneFile);
            }
       }
    }
    if(closedir(cdir)) throw runtime_error(dir + "couldn't close");
}
