/***************************************************************************
                          gfile.h  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GFILE_H
#define GFILE_H

#include <pthread.h>

#include <string>
#include <map>

using namespace std;


/**Manage the gene files and deliver them to the Web server
  *@author Rudiger Koch
  */

class GFile {
public: 
    GFile();
    ~GFile();
    
    /** check out a gene for evaluation. The return value is the Id
        of the gene. Checking out means moving a gene from the
        outgoing to the out directory. */
    string Checkout();

    /** Only one thread may access the genefiles at any given time. This is inefficient but should be OK for now */
    static void Lock();
    /** remove the lock  */
    static void Unlock();
    static void InitLock();

    /** Check in a gene means moving the file from the out to the
        incoming dir and append a line to the fitness file */
    void Checkin(int score);
    /** Construct a gene file with id */
     GFile(int id);
    /** Writes the genome to a file. The filename is specified by
        the ID. */
    void Save(string genome);
    /** Load a genome string from a file */
    string Load();
    /** Select the fitest individuals in a generation and delete
        the rest. If there are all time best individuals, move
        them into the 'best' directory */
    int * SelectFitest();
    /** return the number of genes in the incoming directory */
    int CountIncoming();
    /** This function should be called when there is no other way
        to get genes into the outgoing dir. It copies one of
        the genes in the out dir. */
    void CopyGene();

    static int FitnessCompare(const void* i1, const void* i2);
    /** Increment the counter */
    static int IncrementCounter();
    /** remove the file from the incoming dir */
    void Die();

    /** Read property of int id. */
    const int& GetId();
    
    /** remove all files from a directory */
    static void EmptyDir(string dir);

protected:

    int id;
    struct Individual {
        int id;
        int score;
        bool operator < (const Individual& individual) const
        {
            return (individual.score<score);
        }
    };
    
    static pthread_mutex_t mut_file;


protected: // Protected methods
  /** Return the filename of a gene in directory dir */
    string GetNextGene(string dir);;
    static void Copy(string srcfile, string dstfile);
};

#endif
