/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sun Nov 25 19:13:52 CET 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <pthread.h>
#include <fcntl.h>

#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <iostream>
#include <fstream>
#include <string>
#include <stdexcept>

#include "genome.h"
#include "gfile.h"
#include "amygdala/logging.h"



void SigHandler(int signum){
    cout << "Caught signal: " << signum << endl;
}

void Reset()
{
    string geneserverDir = Genome::GetAttribute("geneserver_dir");
    LOGGER(2, "Resetting " << geneserverDir);

    system((string("rm -rf ") + geneserverDir + "/best" ).c_str());
    system((string("rm -rf ") + geneserverDir + "/out").c_str());
    system((string("rm -rf ") + geneserverDir + "/outgoing").c_str());
    system((string("rm -rf ") + geneserverDir + "/incoming").c_str());
    system((string("mkdir -p ") + geneserverDir + "/best").c_str());
    system((string("mkdir -p ") + geneserverDir + "/out").c_str());
    system((string("mkdir -p ") + geneserverDir + "/outgoing").c_str());
    system((string("mkdir -p ") + geneserverDir + "/incoming").c_str());
    string counterFileName = geneserverDir + string("/counter");
    ofstream counterStream(counterFileName.c_str());
    if(!counterStream.good()){
        cerr << "Cannot open counter file " << counterFileName << endl;
        return;
    }
    counterStream << 0;
    counterStream.close();
}


string HeaderContent(string request, string header){
    string::size_type id_start = request.find(header) + header.length();
    if(id_start == string::npos) throw runtime_error("No Gene-Id Header");
    string::size_type id_end   = request.find("\r\n", id_start);
    return request.substr(id_start, id_end - id_start);
}

void* ServeClient( void* p_socket )
{
    string reqStr    = "GET /genome HTTP/";
    string resultStr = "GET /result HTTP/";
    string uploadStr = "PUT /upload HTTP/";
    int socket = (int)p_socket;
    char buffer[256];
    string request;
    int numbytes;

    pthread_detach(pthread_self());
    while((numbytes = read(socket, buffer, 255 )) > 0) {
        LOGGER(5, "Read " << numbytes << " from client");
        for(int i=0; i<numbytes; i++)
                request.push_back(buffer[i]);
        // The rest of the loop is to figure out if we break the loop
        if(request.substr(0,3) == "GET"){
            if(request.substr(request.length()-4) == "\r\n\r\n") break;
        } else if( request.substr(0,3) == "PUT"){
            string::size_type lenPos = request.find("Content-Length:");
            if(lenPos != string::npos) {
                lenPos += string("Content-Length:").length();
                unsigned int contentLength = atoi(request.substr(lenPos).c_str());
                string::size_type genomePos = request.find("\r\n\r\n");
                if(genomePos != string::npos){ // we have a file - now check it's length
                    genomePos += 4; // set offset to begin of genome
                    string genome = request.substr(genomePos);
                    if(genome.length() > contentLength){
                        cerr << "submitted genome too long: " << genome.length() << endl;
                    } else if(genome.length() < contentLength) {
                        cerr << "submitted genome too short: " << genome.length() << endl;
                    } else {
                        break;
                    }
                }
            }
        } else {
            cerr << "Unknown HTTP method: " << request.substr(0,3) << endl;
            return NULL;
        }
    }

    if(request.length() == 0){
        cerr << "Error: No request was sent" << endl;
        close(socket);
        return NULL;
    }

    GFile::Lock();
    if(request.substr(0, reqStr.size()) == reqStr){
        string genome;
        GFile f;
        try {
            genome = f.Checkout();
            sprintf(buffer, "HTTP/1.0 200 OK\r\nGenome-Id: %d\r\nConnection: close\r\nContent-Type: application/amygdala-genome\r\n\r\n", f.GetId());
            write(socket, buffer, strlen(buffer));
            write(socket, genome.c_str(), genome.length());
            cout << "genome " << f.GetId() << endl;
        } catch (runtime_error e) {
            sprintf(buffer, "HTTP/1.0 404 %s\r\nConnection: close\r\nContent-Type: text/html\r\n\r\n", e.what());
            write(socket, buffer, strlen(buffer));
        } catch (...){
            cerr << __FILE__ << ":" << __LINE__ << "Unknown exception" << endl;
        }
    } else if (request.substr(0, resultStr.size()) == resultStr) {
        string idString;
        string scoreString;
        try {
            idString = HeaderContent(request, "Genome-Id:");
            scoreString = HeaderContent(request, "Score:");
            GFile f(atoi(idString.c_str()));
            f.Checkin(atoi(scoreString.c_str()));
    
            strcpy(buffer, "Connection: close\r\nContent-Type: text/html\r\n\r\n");
            write(socket, buffer, strlen(buffer));
            strcpy(buffer, "Accepted, thank you\r\n");
            write(socket, buffer, strlen(buffer));
            cout << "Got result of " << idString << ". Score: " << scoreString << endl;
        } catch (runtime_error e){
            GFile::Unlock();
            cerr << "Wrong header format: " << e.what() << endl;
            close(socket);
            return NULL ;
        } catch (...){
            cerr << __FILE__ << ":" << __LINE__ << "Unknown exception" << endl;
        }
    } else if (request.substr(0, uploadStr.size()) == uploadStr) {
        string::size_type crlfcrlf = request.find("\r\n\r\n");
        if(crlfcrlf == string::npos){
            cerr << "Wrong request format: " << request << endl;
            return NULL;
        }

        string genomeStr = request.substr(crlfcrlf + 4);
        strcpy(buffer, "Connection: close\r\nContent-Type: text/html\r\n\r\nGot genome, thank you");
        GFile f(GFile::IncrementCounter());
        f.Save(genomeStr);
        write(socket, buffer, strlen(buffer));
    } else {
        cerr << "No such location" << endl;
        try{
            strcpy(buffer, "HTTP/1.0 404 Not found\r\nConnection: close\r\nContent-Type: text/html\r\n\r\n");
            if(write(socket, buffer, strlen(buffer)) <= 0) throw runtime_error("Cannot write to socket");
            strcpy(buffer, "<html><H1>Error 404 Not found</H1></html>");
            if(write(socket, buffer, strlen(buffer)) <= 0) throw runtime_error("Cannot write to socket");
        } catch (runtime_error e){
            GFile::Unlock();
            cerr << e.what() << endl;
            close(socket);
            return NULL;
        } catch (...){
            cerr << __FILE__ << ":" << __LINE__ << "Unknown exception" << endl;
        }

    }
    GFile::Unlock();
    close(socket);
    return NULL;
}



void HttpServer()
{
    pthread_t serverThread;
    int accepted_socket;
    struct sockaddr_in port_addr, accept_addr;

    string s_port = Genome::GetAttribute(string("http_port"));
    uint16_t port = 10080;
    if(s_port != ""){
        port = (uint16_t)atoi(Genome::GetAttribute("http_port").c_str());
    }

    int set_opt = 1;

    fd_set read_set;
    int ready_fd;


    port_addr.sin_family = AF_INET;
    port_addr.sin_addr.s_addr = htonl( INADDR_ANY );
    port_addr.sin_port = htons(port);

    int srv_socket = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );

    setsockopt( srv_socket, SOL_SOCKET, SO_REUSEADDR, (char*)&set_opt, sizeof( set_opt ) );
    if(bind( srv_socket, (struct sockaddr*)&port_addr, sizeof( port_addr ) ) ){
        cerr << "Fatal: Cannot bind socket to port " << port << endl;
        exit (-1);
    } else {
        cout << "Listening to port " << port << endl;
    }

    while(1) {
        listen( srv_socket, 15 );

        FD_ZERO( &read_set );
        FD_SET( srv_socket, &read_set );

        do {
            ready_fd = select( srv_socket+1, &read_set, NULL, NULL, NULL );
        }
        while ( ready_fd <= 0  ||  !FD_ISSET( srv_socket, &read_set ) );

        socklen_t size = sizeof( accept_addr );
        accepted_socket = accept( srv_socket, (struct sockaddr*)&accept_addr, &size );

//        ServeClient((void*)accepted_socket );  // facilitates debugging
        if(pthread_create( &serverThread, NULL, ServeClient, (void*)accepted_socket )) cerr << "Cannot create thread" << endl;
    }
}

void usage(){
    cout << "Usage: gene [-m|g] configfile\n\n" <<
            "    -r : reset the spool directories and exit\n" <<
            "         this is deleting all genomes in the existing spool\n" <<
            "    -p : print config and exit\n" <<
            "    -h : print this help\n\n" <<
            "Without options, go into server mode\n" << endl;
}

int main(int argc, char *argv[])
{

    srand(time(NULL));
    string configfile(argv[argc-1]);
    try {
        Genome::LoadConfig();       
    } catch (runtime_error e) {
        cerr << "An error occurred: " << e.what() << endl;
        exit(-1);
    }
    int c;
    if ((c = getopt(argc, argv, "rph")) != EOF) {
        switch (c) {
            case 'r':
                Reset();
                return EXIT_SUCCESS;
                break;
            case 'p':
                Genome::PrintConfig();
                return EXIT_SUCCESS;
                break;
            case 'h':
                usage();
                return EXIT_SUCCESS;
                break;
            default:
                usage();
                return -1;
        }
    }
    signal(SIGPIPE, SigHandler);
    signal(SIGHUP, SigHandler);
    signal(SIGCHLD, SigHandler);
    signal(SIGINT, exit);

    HttpServer();
    return EXIT_SUCCESS;
}


