#include <iostream>
#include <fstream>
#include <cstdlib>
#include <stdexcept>

#include <amygdala/network.h>
#include <amygdala/inputneuron.h>
#include <amygdala/lsmtopology.h>
#include <amygdala/factory.h>
#include <amygdala/utilities.h>
#include <amygdala/logging.h>
#include <amygdala/scheduledspikeinput.h>
#include <amygdala/alphaneuron.h>
#include <amygdala/pdeltatrainer.h>
#include <amygdala/basicspikeoutput.h>
#include "musspikeinput.h"


using namespace std;
using namespace Amygdala;

vector <float> eventTime;
//MusSpikeInput * spikeInput;
ScheduledSpikeInput* spikeInput;
PDeltaTrainer* mouseTrainer;
AmTimeInt ioPeriod=20000;

void usage(const char * prog)
{
    cout << "Usage:" << endl << endl;
    cout << "        " << prog << " inputfilename" << endl;
    exit(-1);
}



void LoadEvents(const char * filename)
{
    // open filename (should be 40 lines representing 40 "channels")

    // for each line (an event time in seconds), convert
    // the value to us and call spikeInput->TriggerSpikeTrain(...)
    // for each neuron in the channel (generally 20).  If the line is
    // NaN, skip it and move on to the next.

    ifstream fin(filename);
    if (!fin) {
        throw runtime_error(string("Could not open ") + filename);
    }

    char buf[16];
    if (eventTime.size()) {
        eventTime.clear();
    }
    eventTime.reserve(45);

    while (fin.getline(buf, 15)) {
        if (!strcmp(buf, "NaN")) {
            eventTime.push_back(0);
            continue;
        }
        float etime = atof(buf);
        etime *= 1000000.0; // convert to us
        eventTime.push_back(etime);
    }
    fin.close();

    for (uint i=0; i<eventTime.size(); ++i) {
        LOGGER(2,"Channel " << i+1 << ": " << eventTime[i]);
    }
}

void ScheduleInput()
{
    AmIdInt nId;
//    for (unsigned int i=0; i<eventTime.size(); ++i) {
    for (unsigned int i=0; i<40; ++i) {
        AmTimeInt eTime = AmTimeInt(eventTime[i]);
        if (eTime == 0) // no events for this band
            continue;

        InputNeuron* inNrn = dynamic_cast <InputNeuron *> (spikeInput->GetNeuron(i));
        if(inNrn == NULL) throw runtime_error(string("Cannot get InputNeuron") + Utilities::itostr(i));
        spikeInput->ScheduleSpike(inNrn, eTime);
    }
    //spikeInput->GetReadyForRun();
}

int main(int argc, char *argv[])
{
    Network::Init(argc, argv);
    if(argc != 2) usage(argv[0]);
    try{
        LoadEvents(argv[1]);
        
        /*FactoryBase::GetRegistry().AddFactory("MusSpikeInput", new SpikeInputFactory <MusSpikeInput> ());
        SIFactory * sif =  dynamic_cast<SIFactory*> (FactoryBase::GetRegistry().GetFactory("MusSpikeInput"));
        spikeInput = dynamic_cast <MusSpikeInput *> (sif->MakeSpikeInput("mus input"));
        
        TFactory * lsmFactory = dynamic_cast<TFactory*> (FactoryBase::GetRegistry().GetFactory("LSMTopology"));
        LSMTopology *top = dynamic_cast<LSMTopology*> (lsmFactory->MakeTopology("LSM Mouse"));
        top->MakeCube("BasicNeuron", 15, 3, 3);
        top->MakeInput(spikeInput, 40);
        top->MakeReadout("BasicNeuron", 40);*/
        spikeInput = Factory::MakeScheduledSpikeInput("Mouse Input");
        LSMTopology *top = Factory::MakeLSMTopology("LSM Mouse");
        AlphaNeuronProperties nProps(true);
        nProps.SetMembraneConst(30.0);
        nProps.SetExcitatorySynapseConst(3.0);
        nProps.SetInhibitorySynapseConst(6.0);
        AlphaNeuron::Factory makeAlphaNrn;
        top->MakeCube(&makeAlphaNrn, &nProps, 15, 3, 3);
        top->MakeInput(spikeInput, 40);
        
        // set nProps for the readout neurons
        nProps.SetMembraneConst(30.0);
        nProps.SetExcitatorySynapseConst(3.0);
        nProps.SetInhibitorySynapseConst(3.0);
        PDeltaTrainerProperties pdeltaProps(0.005, 0.02, ioPeriod, PDSCALED);
        mouseTrainer = Factory::MakePDeltaTrainer(pdeltaProps, top->MakeReadout(&makeAlphaNrn, &nProps, 25), "mouse pdelta");
        
		// synapse values for all possible combinations of inhibitory / excitatory
        // values from Maass et al. "A Model for Real Time Computation in Neural Microcircuits"
        const LSMTopology::EI A = {30.*1.3, 60.*1.3, -19.*1.3, -19.*1.3};      // scaling param in nA
/* values of Maass */
//        const LSMTopology::EI A = {30., 60., -19, -19};      // scaling param in nA
        const LSMTopology::EI delay = {1500, 800, 800, 800};
        const LSMTopology::EI C = {0.3, 0.2, 0.4, 0.1};      // likelyhood of connection

        const LSMTopology::EI U = {0.5, 0.05, 0.25, 0.32};   // utilization as fraction of 1
        const LSMTopology::EI D = {1.1, 0.125, 0.7, 0.144};  // time const in sec
        const LSMTopology::EI F = {0.05, 1.2, 0.02, 0.06};   // time const in sec
        top->Connect(2.0, A, delay, C, U, D, F, "DynamicSynapse");

        OutputManager::EnableOutput(true, LSMTopology::INPUTGROUP);
//        OutputManager::EnableOutput(true, LSMTopology::OUTPUTGROUP);
        OutputManager::EnableOutput(true, LSMTopology::HIDDENGROUP);

        ScheduleInput();
        BasicSpikeOutput* bso = Factory::MakeBasicSpikeOutput("output");
        bso->LogSpikeTimes("lsmspikes.log", 0, 100e6);
        Network::GetNetworkRef()->InitRun();
        Network::GetNetworkRef()->Run(AmTimeInt(3E6));
    } catch (runtime_error e){
        cerr << "Caught Runtime Error: " << e.what() << endl;
        cerr << "Exiting." << endl;
        exit(-1);
    }
    return EXIT_SUCCESS;
}

