#include <iostream>
#include <fstream>
#include <cstdlib>
#include <stdexcept>

#include <amygdala/network.h>
#include <amygdala/alphaneuron.h>
#include <amygdala/inputneuron.h>
#include <amygdala/lsmtopology.h>
#include <amygdala/factory.h>
#include <amygdala/utilities.h>
#include <amygdala/logging.h>
#include <amygdala/pdeltatrainer.h>
#include <amygdala/basicspikeoutput.h>

#include "lsmspikeinput.h"


using namespace std;
using namespace Amygdala;

vector<unsigned int> inputRatesPos;
//vector<unsigned int> inputRatesNeg;
LsmSpikeInput* posInput;
LsmSpikeInput* negInput;
PDeltaTrainer* posTrainer;
PDeltaTrainer* negTrainer;
AmTimeInt ioPeriod=20000;    // length of input/output/training period -- default to 20ms

void usage(const char * prog)
{
    cout << "Usage:" << endl << endl;
    cout << "        " << prog << " inputfilename" << endl;
    exit(-1);
}

void LoadRates(const char * filename)
{
    ifstream fin(filename);
    if (!fin) {
        throw runtime_error(string("Could not open ") + filename);
    }

    char buf[16];
    while (fin.getline(buf, 15)) {
        int rate = atoi(buf);
        inputRatesPos.push_back(rate);
    }
    fin.close();
}

void InitSpikeInput()
{
    posInput->SetInputPeriod(ioPeriod);
    //negInput->SetInputPeriod(ioPeriod);
    for (unsigned int i=0; i<inputRatesPos.size(); ++i) {
        LsmSpikeInput::PeriodInput pi;
        pi.step = i;
        pi.value = inputRatesPos[i];
        posInput->AddInput(pi);
        
        //pi.value = inputRatesNeg[i];
        //negInput->AddInput(pi);
    }
    for (unsigned int i=0; i<inputRatesPos.size(); ++i) {
        LsmSpikeInput::PeriodInput pi;
        pi.step = i+inputRatesPos.size();
        pi.value = inputRatesPos[i];
        posInput->AddInput(pi);
    }
}

void InitSwingInput()
{
    for (unsigned int i=0; i<500; ++i) {
        for (unsigned int j=0; j<5; ++j) {
            negInput->SwingPattern(j+(i*10));
        }
        for (unsigned int j=0; j<5; ++j) {
            negInput->StraightPattern((j+5)+(i*10));
        }
    }
}

void InitSwingTrainers()
{
    for (unsigned int i=0; i<500; ++i) {
        for (unsigned int j=0; j<5; ++j) {
            PDeltaTrainer::TrainingExample te;
            te.step = j+(i*10);
            te.value = 1;
            posTrainer->AddTrainingExample(te);
        }
        for (unsigned int j=0; j<5; ++j) {
            PDeltaTrainer::TrainingExample te;
            te.step = (j+5)+(i*10);
            te.value = 0;
            posTrainer->AddTrainingExample(te);
        }
    }
}

void InitTrainers()
{
    for (unsigned int i=50; i<inputRatesPos.size(); ++i) {
        PDeltaTrainer::TrainingExample te;
        te.step = i;
        /*if ((inputRatesPos[i-1] + inputRatesPos[i-2] + inputRatesPos[i-3]) > 5) {
            te.value = 1;
        }
        else {
            te.value = 0;
        }*/
        te.value = (float)inputRatesPos[i-1]/130.; // + inputRatesNeg[i];
        /*if (inputRatesPos[i]>inputRatesPos[i-1]) {
            te.value = 1;
        }
        else {
            te.value = 0;
        }*/
        posTrainer->AddTrainingExample(te);
        
        //te.value = inputRatesNeg[i];
        //negTrainer->AddTrainingExample(te);
    }
    for (unsigned int i=2; i<inputRatesPos.size(); ++i) {
        PDeltaTrainer::TrainingExample te;
        te.step = i+inputRatesPos.size();
        te.value = (float)inputRatesPos[i-1]/130.; // + inputRatesNeg[i];
        /*if (inputRatesPos[i]>inputRatesPos[i-1]) {
            te.value = 1;
        }
        else {
            te.value = 0;
        }*/
        posTrainer->AddTrainingExample(te);
    }
}

void ScheduleInput()
{
/*    AmIdInt nId;
//    for (unsigned int i=0; i<eventTime.size(); ++i) {
    for (unsigned int i=0; i<40; ++i) {
        AmTimeInt eTime = AmTimeInt(eventTime[i]);
        if (eTime == 0) // no events for this band
            continue;

        InputNeuron* inNrn = dynamic_cast <InputNeuron *> (spikeInput->GetNeuron(i));
        if(inNrn == NULL) throw runtime_error(string("Cannot get InputNeuron") + Utilities::itostr(i));
        spikeInput->TriggerSpikeTrain(inNrn, eTime);
    }
    spikeInput->GetReadyForRun();*/
}

int main(int argc, char *argv[])
{
    Network::Init(argc, argv);
    if(argc != 2) usage(argv[0]);
    try{
        LoadRates(argv[1]);
        
        /*Factory::GetRegistry().AddFactory("MusSpikeInput", new SpikeInputFactory <MusSpikeInput> ());
        SIFactory * sif =  dynamic_cast<SIFactory*> (Factory::GetRegistry().GetFactory("MusSpikeInput"));*/
        posInput = Factory::MakeLsmSpikeInput("pos input");
        //negInput = Factory::MakeLsmSpikeInput("neg input");
        
        //TFactory * lsmFactory = dynamic_cast<TFactory*> (Factory::GetRegistry().GetFactory("LSMTopology"));
        LSMTopology *top = Factory::MakeLSMTopology("LSM Counter");
        //top->MakeCube("BasicNeuron", 15, 3, 3);
        AlphaNeuronProperties nProps(true);
        nProps.SetMembraneConst(30.0);
        nProps.SetExcitatorySynapseConst(3.0);
        nProps.SetInhibitorySynapseConst(6.0);
        AlphaNeuron::Factory makeAlphaNrn;
        top->MakeCube(&makeAlphaNrn, &nProps, 15, 3, 3);
        top->MakeInput(posInput, 5);
        //top->MakeInput(negInput, 5);
        
        // set nProps for the readout neurons
        nProps.SetMembraneConst(30.0);
        nProps.SetExcitatorySynapseConst(3.0);
        nProps.SetInhibitorySynapseConst(3.0);
        PDeltaTrainerProperties pdeltaProps(0.005, 0.02, ioPeriod, PDSCALED);
        posTrainer = Factory::MakePDeltaTrainer(pdeltaProps, top->MakeReadout(&makeAlphaNrn, &nProps, 25), "pos pdelta");
        //negTrainer = Factory::MakePDeltaTrainer(pdeltaProps, top->MakeReadout(&makeAlphaNrn, &nProps, 5), "neg pdelta");
        
        // synapse values for all possible combinations of inhibitory / excitatory
        // values from Maass et al. "A Model for Real Time Computation in Neural Microcircuits"
        //const LSMTopology::EI A = {30./19, 60./19, -19/19, -19/19};      // scaling param in nA
/* values of Maass */
        //const LSMTopology::EI A = {30., 60., -19, -19};      // scaling param in nA
        const LSMTopology::EI A = {30.*1.3, 60.*1.3, -19.*1.3, -19.*1.3};      // scaling param in nA
        const LSMTopology::EI delay = {1500, 800, 800, 800};
        const LSMTopology::EI C = {0.3, 0.2, 0.4, 0.1};      // likelyhood of connection

        const LSMTopology::EI U = {0.5, 0.05, 0.25, 0.32};   // utilization as fraction of 1
        const LSMTopology::EI D = {1.1, 0.125, 0.7, 0.144};  // time const in sec
        const LSMTopology::EI F = {0.05, 1.2, 0.02, 0.06};   // time const in sec
        top->Connect(2.0, A, delay, C, U, D, F, "DynamicSynapse");

        OutputManager::EnableOutput(false, LSMTopology::INPUTGROUP);
        OutputManager::EnableOutput(false, LSMTopology::OUTPUTGROUP);
        OutputManager::EnableOutput(false, LSMTopology::HIDDENGROUP);
        
        InitSpikeInput();
        InitTrainers();
        //InitSwingInput();
        //InitSwingTrainers();
        BasicSpikeOutput* bso = Factory::MakeBasicSpikeOutput("output");
        bso->LogSpikeTimes("lsmspikes.log", 0, 100e6);
        Network::GetNetworkRef()->InitRun();
        //ScheduleInput();
        Network::GetNetworkRef()->Run(AmTimeInt(450*1E6));
        //posTrainer->PrintWeights();
    } catch (runtime_error e){
        cerr << "Caught Runtime Error: " << e.what() << endl;
        cerr << "Exiting." << endl;
        exit(-1);
    }
    return EXIT_SUCCESS;
}

