/***************************************************************************
                          lsmspikeinput.cpp  -  description
                             -------------------
    copyright            : (C) 2005 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>
#include <fstream>
#include <iostream>
#include <amygdala/inputneuron.h>
#include <amygdala/utilities.h>
#include <amygdala/logging.h>
#include "lsmspikeinput.h"

using namespace std;
using namespace Amygdala;

LsmSpikeInput::LsmSpikeInput(): SpikeInput()
{
}

LsmSpikeInput::LsmSpikeInput(const string& name): SpikeInput(name), inputPeriodLength(30000), lastSpike(0), inputPeriod(0)
{
}

LsmSpikeInput::~LsmSpikeInput()
{
}

void LsmSpikeInput::AddNeuron(InputNeuron* nrn)
{
	nrnMap[nrn->GetId()] = nrn;
	NeuronGroup::AddNeuron(nrn); 
}

void LsmSpikeInput::AddInput(const PeriodInput& input)
{
    AmTimeInt baseTime = input.step * inputPeriodLength;
    AmTimeInt spikePeriod = 1e6/input.value;
    spikePeriod /= 100;    // round off to nearest 0.1 ms
    spikePeriod *= 100;
    
    // TODO: Add jitter to spike times
    for (AmTimeInt i=0; i<inputPeriodLength; i+=spikePeriod) {
        AmTimeInt nextSpike = lastSpike + spikePeriod;
        if (nextSpike < baseTime) nextSpike = baseTime;
        lastSpike = nextSpike;
        InputNeuron* nrn = dynamic_cast<InputNeuron*>(nrnGroup[0]);
        ScheduleSpike(nrn, nextSpike);
    }
    // generate times between 0 and inputPeriodLength
    // divide the time by 100 and the multiply by 100 to round
    // it off properly
    /*for (unsigned int i=0; i<input.value; ++i) {
        AmTimeInt spikeTime = rand()%inputPeriodLength;
        spikeTime /= 100;
        spikeTime *= 100;
        spikeTime += baseTime;
        unsigned int nrnIdx = rand()%nrnGroup.size();
        InputNeuron* nrn = dynamic_cast<InputNeuron*>(nrnGroup[nrnIdx]);
        ScheduleSpike(nrn, spikeTime);
    }*/
}

void LsmSpikeInput::SwingPattern(unsigned int cycle)
{
    AmTimeInt baseTime = cycle*20000;
    InputNeuron* nrn = dynamic_cast<InputNeuron*>(nrnGroup[0]);
    ScheduleSpike(nrn, baseTime+5000);
    ScheduleSpike(nrn, baseTime+10000);
}

void LsmSpikeInput::StraightPattern(unsigned int cycle)
{
    AmTimeInt baseTime = cycle*20000;
    InputNeuron* nrn = dynamic_cast<InputNeuron*>(nrnGroup[0]);
    ScheduleSpike(nrn, baseTime+10000);
    //ScheduleSpike(nrn, baseTime+15000);
}

void LsmSpikeInput::ScheduleSpike(InputNeuron* nrn, AmTimeInt spikeTime)
{
	LOGGER(6, "Scheduling spike from neuron " << nrn->GetId() << " at " << spikeTime)
	InSpike inSpike;
	inSpike.inNeuron = nrn;
	inSpike.spikeTime = spikeTime;
	inputSpike.push_back(inSpike);
}

void LsmSpikeInput::ReadInputBuffer()
{
	LOGGER(6, "Reading input buffer")
	ScheduleQueuedSpikes();
}

