/***************************************************************************
                          lsmspikeinput.h  -  description
                             -------------------
    copyright            : (C) 2005 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LSMSPIKEINPUT_H
#define LSMSPIKEINPUT_H

#include <amygdala/spikeinput.h>
#include <amygdala/factory.h>
#include <map>
#include <vector>

namespace Amygdala {
class LsmSpikeInput: public SpikeInput {
public:
    typedef SpikeInputFactory<LsmSpikeInput> Factory;
	virtual ~LsmSpikeInput();
    
    struct PeriodInput {
        unsigned int step;
        unsigned int value;
        bool operator<(const PeriodInput& rhs) const {
            return step>rhs.step;
        }
    };
    
    void SwingPattern(unsigned int cycle);
    void StraightPattern(unsigned int cycle);

	virtual void AddNeuron(InputNeuron* nrn);
	virtual void ReadInputBuffer();

    void AddInput(const PeriodInput& input); // { inputs.push_back(input); }
	void ScheduleSpike(InputNeuron* nrn, AmTimeInt spikeTime);
    
    void SetInputPeriod(AmTimeInt p) { inputPeriodLength = p; }
    AmTimeInt GetInputPeriod() const { return inputPeriodLength; }
protected:
    LsmSpikeInput(const std::string& name);
	LsmSpikeInput();
	
    AmTimeInt inputPeriodLength;
    AmTimeInt lastSpike;
    unsigned int inputPeriod;
	std::map<AmIdInt, InputNeuron*> nrnMap;
    //std::vector<PeriodInput> inputs;
    
    template<class spikeinput>
    friend class SpikeInputFactory;
};

namespace Factory {
    static LsmSpikeInput::Factory MakeLsmSpikeInput;
}
}  // namespace Amygdala

#endif
