/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fri Mar  1 21:12:27 CET 2002
    copyright            : (C) 2002 by Rudiger Koch, Matt Grover
    email                : rkoch@rkoch.org, matt@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/**@author Rdiger Koch <rkoch@rkoch.org>
  *@author Matt Grover <matt@amygdala.org> */

#include <amygdala/network.h>

#include "mouse.h"
#include "defaultdata.h"
#include <string>
#include <iostream>

using namespace std;
using namespace Amygdala;

int ParseCommandLine(int argc, char *argv[]);
void Usage(char *cmd);

string trainingFile;
string testFile;
float maxSpikeRate=115.0;
float synapticConst=2.0, membraneConst=20.0;
float areaAWeight=0.1, areaWWeight=0.042;

int main(int argc, char *argv[])
{
    Network::Init(argc, argv);
    bool useFileData = true;
    if(argc < 2) useFileData=false;
    cout << "Mouse started...\n";
    ParseCommandLine(argc, argv);
    cout << "Creating mouse...\n";
    Mouse mouse(maxSpikeRate, areaAWeight, areaWWeight, synapticConst, membraneConst);
    try{
        if (useFileData) {
            //cout << "Calling LoadEvents()...\n";
            mouse.LoadEvents(trainingFile);
            //cout << "Calling Train2()...\n";
            mouse.Train2();
            //cout << "Loading test file events...\n";
            mouse.LoadEvents(testFile);
        }
        else {
            mouse.LoadEvents(exemplarData);
            mouse.Train2();
            mouse.LoadEvents(testData);
        }
        mouse.Run();
    } catch (string e) {
        cout << e << endl;
        return EXIT_FAILURE;
    }
    cout << "\nWord recognition is indicated by three or more Gamma neuron spikes\n";
    cout << "spaced 16 to 33 milliseconds apart (30 - 60 Hz). If no spikes were\n";
    cout << "reported above, then the test pattern was not recognized as the trained word.\n";
    return EXIT_SUCCESS;
}

int ParseCommandLine(int argc, char *argv[])
{
    int idx;
    for (idx=1; idx<argc; ++idx) {
        // break if the argument does not start with '-'
        if (strncmp(argv[idx], "-", 1))
            break;

        string arg = argv[idx];
        if (arg == "--help") {
            Usage(argv[0]);
            break;
        }

        if ((arg == "-w") || (arg.substr(0, 11) == "--w-weight=")) {
            string val;
            if (arg == "-w") {
                if (argc < ++idx)
                    Usage(argv[0]);

                val = argv[idx];
            }
            else {
                val = arg.substr(11);
            }
            if ( !(areaWWeight = atof(val.c_str())) ) {
                if ((val == "0") || (val == "0.0")) {
                    areaWWeight = 0.0;
                }
                else {
                    cout << val << " is not numeric.\n";
                    Usage(argv[0]);
                }
            }

            // weight must be in range (0-1)
            if ( (areaWWeight>1.0) || (areaWWeight<0.0) ) {
                cout << "Weights must be in the range (0-1).\n";
                Usage(argv[0]);
            }
        }
        else if ((arg == "-a") || (arg.substr(0, 11) == "--a-weight=")) {
            string val;
            if (arg == "-a") {
                if (argc < ++idx)
                    Usage(argv[0]);

                val = argv[idx];
            }
            else {
                val = arg.substr(11);
            }
            if ( !(areaAWeight = atof(val.c_str())) ) {
                cout << val << " is not numeric.\n";
                Usage(argv[0]);
            }

            // weight must be in range (0-1)
            if ( (areaAWeight>1.0) || (areaAWeight<0.0) ) {
                cout << "Weights must be in the range (0-1).\n";
                Usage(argv[0]);
            }
        }
        else if ((arg == "-s") || (arg.substr(0, 17) == "--synaptic-const=")) {
            string val;
            if (arg == "-s") {
                if (argc < ++idx)
                    Usage(argv[0]);

                val = argv[idx];
            }
            else {
                val = arg.substr(17);
            }
            if ( !(synapticConst = atof(val.c_str())) ) {
                cout << val << " is not numeric.\n";
                Usage(argv[0]);
            }

            // time constant must be in range (1-30)
            if ( (synapticConst>30.0) || (synapticConst<1.0) ) {
                cout << "Time constants must be in the range (1-30).\n";
                Usage(argv[0]);
            }
        }
        else if ((arg == "-m") || (arg.substr(0, 17) == "--membrane-const=")) {
            string val;
            if (arg == "-m") {
                if (argc < ++idx)
                    Usage(argv[0]);

                val = argv[idx];
            }
            else {
                val = arg.substr(17);
            }
            if ( !(membraneConst = atof(val.c_str())) ) {
                cout << val << " is not numeric.\n";
                Usage(argv[0]);
            }

            // time constant must be in range (1-30)
            if ( (membraneConst>30.0) || (membraneConst<1.0) ) {
                cout << "Time constants must be in the range (1-30).\n";
                Usage(argv[0]);
            }
        }
        else if ((arg == "-r") || (arg.substr(0, 17) == "--max-spike-rate=")) {
            string val;
            if (arg == "-r") {
                if (argc < ++idx)
                    Usage(argv[0]);

                val = argv[idx];
            }
            else {
                val = arg.substr(17);
            }
            if ( !(maxSpikeRate = atof(val.c_str())) ) {
                cout << val << " is not numeric.\n";
                Usage(argv[0]);
            }

            // spike rate must be in range (10-300)
            if ( (maxSpikeRate>300.0) || (maxSpikeRate<10.0) ) {
                cout << "The maximum spike rate must be in the range (10-300).\n";
                Usage(argv[0]);
            }
        }
        else if ((arg == "-n") || (arg.substr(0, 15) == "--neuron-model=")) {
            cout << "The neuron model option is not yet supported.\n";
            Usage(argv[0]);
        }
    }

    if (argc<idx+1) {
        //Usage(argv[0]);
        return 1;
    }
    trainingFile = argv[idx++];
    testFile = argv[idx];

    return 1;
}

void Usage(char *cmd)
{
    cout << "Usage: ";
    cout << cmd << " [options] training_file test_file" << endl;
    cout << "Options:\n"
            "    -w, --w-weight=N           Set a weight of N (0-1) for connections between\n"
            "                               area W neurons. [0.042]\n"
            "    -a, --a-weight=N           Set a weight of N (0-1) connections between\n"
            "                               area A and area W neurons. [0.1]\n"
            "    -s, --synaptic-const=N     Set the synaptic time constant to N (1.0-30.0)\n"
            "                               for all neurons.  Time constants are in\n"
            "                               milliseconds. [2.0]\n"
            "    -m, --membrane-const=N     Set the membrane time constant to N (1.0-30.0)\n"
            "                               for all neurons.  Time constants are in\n"
            "                               milliseconds. [20.0]\n"
            "    -r, --max-spike-rate=N     Set the maximum imput spike rate to N\n"
            "                               spikes/second. [115]\n"
            "    --help                     Display this message.\n";

    exit( EXIT_FAILURE );
}

