/***************************************************************************
                          mouse.cpp  -  description
                             -------------------
    begin                : Thu Apr 4 2002
    copyright            : (C) 2002 by Rudiger Koch, Matt Grover
    email                : rkoch@rkoch.org, matt@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
using namespace std;

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <vector>
#include <string>
#include "mouse.h"
#include <amygdala/flatlayer.h>
#include <amygdala/alphaneuron.h>
#include <amygdala/inputneuron.h>
#include <amygdala/synapse.h>
#include <amygdala/nconnector.h>
#include <amygdala/axon.h>
#include <amygdala/physicalproperties.h>
#include <amygdala/utilities.h>
#include "musspikeinput.h"

using namespace Amygdala;

// uncomment to enable debugging messages
//#define MOUSE_DEBUG

Mouse::Mouse(float _maxSpikeRate,
                float _areaAWeight,
                float _areaWWeight,
                float _synapticConst,
                float _membraneConst):
                neuronsPerBand(20),
                bands(40),
                lastLayerId(1),
                layerCount(1),
                maxSpikeRate(_maxSpikeRate/1000000.0),
                areaAWeight(_areaAWeight),
                areaWWeight(_areaWWeight),
                esynapticConst(_synapticConst),
                isynapticConst(_synapticConst+4.0),
                membraneConst(_membraneConst) {
//////////// API EXAMPLE ////////////////
    // Build a MusSpikeInput object.
	spikeInput = Factory::MakeMusSpikeInput("input");

    // Set up the Alpha, Beta, and Gamma layers (the Area A layers will be done later on).
    // I use a FlatLayer here because I don't need the topology to have any special properties.
    // I just need a container for the neurons (all neurons must belong to a Topology object).
    // Notice that the TopologyFactory is a functor -- the object can be called like a function.
    TopologyFactory<FlatLayer> makeFlatLayer;
    alpha = makeFlatLayer("alpha");
    beta = makeFlatLayer("beta");
    gamma = makeFlatLayer("gamma");
//////////// END ////////////////////////
    
	BuildAreaA();
}


Mouse::~Mouse(){
}

void Mouse::LoadEvents(string filename)
{
    // open filename (should be 40 lines representing 40 "channels")

    // for each line (an event time in seconds), convert
    // the value to us and call spikeInput->TriggerSpikeTrain(...)
    // for each neuron in the channel (generally 20).  If the line is
    // NaN, skip it and move on to the next.

    ifstream fin(filename.c_str());
    if (!fin) {
        throw string("Could not open ") + filename;
    }

    char buf[16];
    if (eventTime.size()) {
        eventTime.clear();
    }
    eventTime.reserve(45);

    while (fin.getline(buf, 15)) {
        if (!strcmp(buf, "NaN")) {
            eventTime.push_back(0);
            continue;
        }
        float etime = atof(buf);
        etime *= 1000000.0; // convert to us
        eventTime.push_back(etime);
    }
    fin.close();

    // print out the event times
    #ifdef MOUSE_DEBUG
    for (uint i=0; i<eventTime.size(); ++i) {
        cout << "Channel " << i+1 << ": " << eventTime[i] << endl;
    }
    #endif
    maxRunTime = 1500000;
}

void Mouse::LoadEvents(float* eventData)
{
    if (eventTime.size()) {
        eventTime.clear();
    }
    eventTime.reserve(45);

    for (unsigned int i=0; i<40; ++i) {
        float etime = eventData[i] * 1000000.0; // convert to us
        eventTime.push_back(etime);
    }
    maxRunTime = 1500000;
}

void Mouse::Train2()
{
    // TODO: This function has to be adjusted to do extra checks
    // if more than one pattern is learned.
    // Find the set of area A neurons that should form connections
    // with area W neurons.  This is a training shortcut (an
    // algorithmic training method).
    cout << "Training...\n";
    vector<InputNeuron*> active = FindActiveNeurons();
    vector<SpikingNeuron*> ab;
    ab.reserve(bands*2);

//////////// API EXAMPLE ////////////////
    // Set up factories that will be needed below
    AlphaNeuronProperties aProps(1);    // set up AlphaNeuronProperties to also initialize PhysicalProperties
    aProps.SetMembraneConst(membraneConst);
    aProps.SetExcitatorySynapseConst(esynapticConst);
    aProps.SetInhibitorySynapseConst(isynapticConst);

    // Each Neuron class has a factory typedefed, so you can just say
    // FooNeuron::Factory and get the right kind of template.  In this case
    // it resolves to NeuronFactory<AlphaNeuron, AlphaNeuronProperties>.
    // As with the TopologyFactory, NeuronFactory is a functor.
    AlphaNeuron::Factory makeAlphaNeuron;

    // Synapses also typedef NeuronConnector templates.
    StaticSynapse::NeuronConnector nc;
    StaticSynapseProperties ssp(areaAWeight, 1000);

    int x = -8;
    int y = 0;

    // Make the connections
    // Connect each active areaA neuron (found in training procedure) to one alpha and
    // one beta neuron.  Note that an alpha neuron here is just an AlphaNeuron in the alpha
    // layer. Neurons in the beta layer are also AlphaNeurons.
    for (unsigned int i=0; i<active.size(); ++i) {
        // connect active[i] to one alpha and one beta neuron.
        // alpha and beta neurons should be created and added to
        // the area W layer. Neuron IDs can be generated by calling
        // net->MaxNeuronId() + 1.
        InputNeuron* aNrn = active[i];
        if (!aNrn->GetAxon()->SynapseCount()) {
            // Set location variables -- used for OpenGL debugger
            if(x == 8) {
                y+=2;
                x=-8;
            }
            x+=2;
            aProps.GetPhysicalProps()->SetPosition(x, y, 0);
            AlphaNeuron* alphaNrn = makeAlphaNeuron(aProps, alpha);     // add a neuron to the alpha layer

            aProps.GetPhysicalProps()->SetPosition(x, y, 4);
            AlphaNeuron* betaNrn = makeAlphaNeuron(aProps, beta);       // add a neuron to the beta layer

            // Keep track of the pointers for later on
            ab.push_back(alphaNrn);
            ab.push_back(betaNrn);

            betaNrn->Inhibitory(true);

            // Add the neurons with a delay of 1000 ms (specified in the props).  Something to
            // try here is to experiment with the delay and see if it
            // makes a difference.
            nc.Connect(aNrn, alphaNrn, ssp);
            nc.Connect(aNrn, betaNrn, ssp);
        }
    }

    // add the gamma neuron.  the neuron id should be greater
    // than 100000 to make identification easier for reporting
    // output
    aProps.GetPhysicalProps()->SetPosition(3, 5, 8);
    aProps.SetExcitatorySynapseConst(2.0);
    aProps.SetInhibitorySynapseConst(6.0);
    aProps.SetMembraneConst(6.0);
    AlphaNeuron* gammaNrn = makeAlphaNeuron(100001, aProps, gamma);

    // create all to all connections between alpha and beta nrns
    // and connect to gamma neuron
    float weight;
    for (unsigned int i=0; i<ab.size(); ++i) {
        if (ab[i]->Inhibitory()) {
            weight = -1.0*areaWWeight;  // was 2.2
    		ssp.SetWeight(-0.053);      // TODO: remove hardcoded weight
            nc.Connect(ab[i], gammaNrn, ssp);
        }
        else {
            weight = areaWWeight;
            ssp.SetWeight(0.068);
            nc.Connect(ab[i], gammaNrn, ssp);
        }
        for (unsigned int j=0; j<ab.size(); ++j) {
            if (i==j)
                continue;

            ssp.SetWeight(weight);
            nc.Connect(ab[i], ab[j], ssp);
        }
    }
//////////////// END ///////////////////
}


vector<InputNeuron*> Mouse::FindActiveNeurons()
{
    // Find the set of area A neurons that should be activated
    // (form connections with area W) for the current training
    // pattern.  Indeces match the indeces for the fbands vector
    // (one active neuron per frequency band).
    vector<InputNeuron*> activeNrn;
    activeNrn.reserve(bands);

    vector<float> decayRate;
    decayRate.reserve(decayTime.size()+1);
    for (unsigned int i=0; i<decayTime.size(); ++i) {
        decayRate.push_back(-maxSpikeRate/decayTime[i]);
        //cout << "rate " << i << ": " << decayRate[i] << endl;
    }

    // Start by finding the maximum and minimum event times
    // and the associated neurons.  The max and min events
    // will be used to find a convergence point.

    float tmin=0.0, tmax=0.0;       // min/max event times
    unsigned int imin=0, imax=0;    // min/max indeces for fband

    tmin = eventTime[0];
    for (unsigned int i=0; i<eventTime.size(); ++i) {
        float et = eventTime[i];
        if (et < tmin) {
            if (et > 0.0) {     // rule out non-events
                tmin = et;
                imin = i;
            }
        }
        if (et > tmax) {
            tmax = et;
            imax = i;
        }
    }

    #ifdef MOUSE_DEBUG
    cout << "Min event time: " << tmin << "\nMax event time: " << tmax << endl;
    cout << "Min event index: " << imin << "\nMax event index: " << imax << endl;
    #endif

    // Assume that the first event has the slowest possible decay rate
    // and the last event has the fastest.  Find the convergence time
    // and the convergence weight.  Change the decay rates until the
    // convergence rate is as close to 0.5 as possible.
    float te1, te2;         // event times
    float td1, td2;         // decay times
    float gamma1, gamma2;   // gamma = te/(te+td)
    float rd1, rd2;         // decay rates
    unsigned int i1, i2;    // indexes for bands
    unsigned int dIdx1, dIdx2;
    float conTime=0.0;      // convergence time
    float minConTime=0.0, lastConTime=0.0;
    float currRate=0.0, lastRate=0.0;

    i1=imin;
    i2=imax;
    dIdx1=decayRate.size() - 1;
    dIdx2= 0;
    rd1=decayRate[dIdx1];    // decay rate ordered steep->shallow
    rd2=decayRate[dIdx2];
    td1=decayTime[dIdx1];
    td2=decayTime[dIdx2];
    te1=eventTime[i1];
    te2=eventTime[i2];

    //cout << "variables: " << rd1 << " " << rd2 << " " << td1 << " ";
    //cout << td2 << " " << te1 << " " << te2 << endl;
    //gamma1 = te1/(te1+td1);
    //gamma2 = te2/(te2+td2);
    gamma1 = rd1*(te1+td1);
    gamma2 = rd2*(te2+td2);
    //cout << "gamma1,2: " << gamma1 << " " << gamma2 << endl;
    //minConTime = (gamma2-gamma1)/(rd1-rd2);
    minConTime = (gamma1-gamma2)/(rd1-rd2);
    currRate = maxSpikeRate + (minConTime-te1)*rd1;
    lastConTime=minConTime;
    lastRate=currRate;
    #ifdef MOUSE_DEBUG
    cout << "Minimum convergence time: " << minConTime << endl;
    cout << "Rate (max): " << currRate * 1000000.0 << endl;
    #endif

    float halfMax = 0.75*maxSpikeRate;
    if (currRate>halfMax) {
        // try for something lower
        while (currRate>halfMax) {
            --dIdx1;
            ++dIdx2;
            if (dIdx1 <= dIdx2) {
                // convergence is not possible for any more
                // combinations
                break;
            }

            lastRate=currRate;
            lastConTime=conTime;
            rd1=decayRate[dIdx1];
            rd2=decayRate[dIdx2];
            td1=decayTime[dIdx1];
            td2=decayTime[dIdx2];
            te1=eventTime[i1];
            te2=eventTime[i2];

            gamma1 = rd1*(te1+td1);
            gamma2 = rd2*(te2+td2);
            conTime = (gamma1-gamma2)/(rd1-rd2);
            currRate = maxSpikeRate + (conTime-te1)*rd1;
        }

        if ((halfMax-currRate) > (lastRate-halfMax)) {
            ++dIdx1;    //reset the indexes to last values
            --dIdx2;
            conTime=lastConTime;    // closer to half max rate than currRate
            currRate = lastRate;
        }
    }
    else {
        conTime=lastConTime;
        currRate=lastRate;
    }

    #ifdef MOUSE_DEBUG
    cout << "Convergence time: " << conTime << endl;
    cout << "Convergence rate: " << currRate*1000000.0 << " spikes/sec.\n";
    #endif
    // We have the convergence time and indexes for the first and last neurons.
    // Add the corresponding neurons to the activeNeurons vector and
    // find convergence rates/neurons for the rest of the bands
    float timeDiff;
    float dTime, delta, lastDelta=0.0; //0xFFFFFFFF;
    unsigned int dIndex;

    for (unsigned int i=0; i<bands; ++i) {
        timeDiff = conTime - eventTime[i];
        dTime = (timeDiff*maxSpikeRate)/(maxSpikeRate-currRate);
        #ifdef MOUSE_DEBUG
        cout << "Channel " << i << " requested decay time: " << dTime << endl;
        #endif

        // find the closest matching area A neuron
        lastDelta = 1.0e10;
        for (unsigned int j=0; j<decayTime.size(); ++j) {
            delta = fabs(dTime - decayTime[j]);
            if ( delta > lastDelta ) {
                // last time was best match
                if (j>0) {
                    dTime = decayTime[j-1];
                    dIndex = j-1;
                }
                else {
                    dTime = decayTime[0];
                    dIndex = 0;
                }
                lastDelta = 1.0e10; //0xFFFFFFFF;
                break;
            }
            lastDelta = delta;
            if (j==(decayTime.size() - 1)) {
                dTime = decayTime[j];
                dIndex = j-1;
            }
        }

        #ifdef MOUSE_DEBUG
        cout << "Channel " << i << " actual decay time: " << dTime << endl;
        #endif
        InputNeuron* aNrn = dynamic_cast<InputNeuron*>((*(areaA[i]))[dIndex]);
        activeNrn.push_back(aNrn);
    }
    return activeNrn;
}

void Mouse::ScheduleInput()
{
    // the event list has to be full
    if (!eventTime.size()) {
        cerr << "Mouse::ScheduleInput() was called before the sound file was loaded.\n";
        return;
    }

    AmIdInt nId;
    for (unsigned int i=0; i<eventTime.size(); ++i) {
        FlatLayer* tmpLayer = areaA[i];
        AmTimeInt eTime = AmTimeInt(eventTime[i]);
        if (eTime == 0) // no events for this band
            continue;

        for (unsigned int j=0; j<(*tmpLayer).size(); ++j) {
            if (!(*tmpLayer)[j]->GetAxon()->SynapseCount()) {
                continue;
            }
            InputNeuron* inNrn = dynamic_cast<InputNeuron*>((*tmpLayer)[j]);
            spikeInput->TriggerSpikeTrain(inNrn, decayTime[j],
                                            eTime);
        }
    }
    spikeInput->GetReadyForRun();
}

void Mouse::InitOutput()
{
///////////// API EXAMPLE ///////////////////
    // Set up the output object.  We need a factory to do this.
    spikeOut = Factory::MakeMouseOutput("output");
    spikeOut->LogSpikeTimes(string("spikes_run.log")); // only applies to StatisticsOutput, the MouseOutput base

    // Here we set up the output groups.  Output groups are just a way to turn output on or off
    // for a group of Neurons.  There are two ways to do this: we can call OutputManager::AddNeuronToGroup()
    // for each neuron that we want in a group.  Or we can call NeuronGroup::AddToOutputGroup() to add
    // all of the Neurons in a given NeuronGroup to an output group.  Topology classes derive from NeuronGroup
    // so we can call that for each layer.
    // Add all area A neurons to group 1, alpha/beta neurons to group 2, and gamma neuron(s) to 3
    for (unsigned int i=0; i<areaA.size(); ++i) {
        areaA[i]->AddToOutputGroup(1);
    }
    alpha->AddToOutputGroup(2);
    beta->AddToOutputGroup(2);
    gamma->AddToOutputGroup(3);

    // check to see if output groups have neurons
    /*OutputManager::output_group_iterator itr;
    for (itr=OutputManager::OutputGroup_begin(); itr!=OutputManager::OutputGroup_end(); itr++) {
        cout << "Output group " << itr->first << " has " << itr->second.size() << " neurons\n";
    }*/

    // Turn on output for groups that should report output
    // turn on output for everything for now
    //OutputManager::EnableOutput(true, 1);  // turns output on. OutputManager::DisableOutput(1) turns it off
	OutputManager::EnableOutput(true, 2);
    OutputManager::EnableOutput(true, 3);

    // This is specific to StatisticsOutput.  It turns on logging for any neurons in a group
    // with a trace.
    spikeOut->AddTrace(1);
    spikeOut->AddTrace(2);
    spikeOut->AddTrace(3);
//////////// END ///////////////////
}    

void Mouse::Run(){
    InitOutput();
/////////// API EXAMPLE ///////////////
    // InitRun() has to be called before the network can do anything.  This should be called
    // after the neurons have been created and connected, but before Run() is called.
    Network::GetNetworkRef()->InitRun();    
    ScheduleInput();
    cout << "Running...\n";
    Network::GetNetworkRef()->Run(maxRunTime);
    spikeOut->CloseLog();
}

void Mouse::BuildAreaA()
{
    // set up the decay rates.  each frequency band will have
    // neuronsPerBand neurons with decay times from 0.3s .. 1.1s.
    // area A has a layer for each band and the set of decay times
    // is the same for all layers.
    decayTime.reserve(neuronsPerBand);
    AmTimeInt decayStepSize;    // us
    AmTimeInt lastDTime;
    decayStepSize = 1000*(1100 - 300)/neuronsPerBand;
    decayTime.push_back(300000);
    lastDTime = 300000;

    for (unsigned int i=1; i<neuronsPerBand; i++) {
        lastDTime += decayStepSize;
        decayTime.push_back(lastDTime);
    }

//////////// API EXAMPLE ///////////////
    InputNeuronProperties inProps;
    InputNeuron::Factory makeInput;
    TopologyFactory<FlatLayer> makeLayer;

    // Set up the input layer
    for (unsigned int i=0; i<bands; ++i) {
        string layerName("Input Layer ");
        layerName += Utilities::itostr(i);

        FlatLayer* fl = makeLayer(layerName);

        for (unsigned int j=0; j<neuronsPerBand; ++j) {
            makeInput(inProps, fl);
        }

        areaA.push_back(fl);
    }
//////////// END ////////////////

    // set up the input bands
    fbands.reserve(bands);
    for(unsigned int f=1; f<=bands; f++){    // 40 freqency bands
        Band tmpBand;

        int percent = 1 + (int)(100.*rand()/(RAND_MAX+1.0));
        if(percent <=35){
            tmpBand.type = BTYPE_ON;
        } else if(percent <= 70){
            tmpBand.type = BTYPE_OFF;
        } else {
            tmpBand.type = BTYPE_PEAK;
        }
        tmpBand.threshold = 1.0;
        tmpBand.ID = f;
        tmpBand.active = false;
        fbands.push_back(tmpBand);
    }
}

float Mouse::RandPercent()
{
    return ( float(rand()) / float(RAND_MAX) ) * 100.0;
}

void MouseOutput::OutputEvent(Neuron* nrn, AmTimeInt eventTime)
{
    if (nrn->GetId()>100000) {
        // gamma neuron
        cout << "Gamma neuron " << nrn->GetId()-100000 << " spiked at " << eventTime/1000
             << " ms.\n";
    }
    else {
        // pass output from the other neurons on to the parent class
        StatisticsOutput::OutputEvent(nrn, eventTime);
        //cout << "neuron " << nrn->GetClassName() << " " << nrn->GetId() << " spiked at " << eventTime/1000 << " ms.\n";
    }
}
