/***************************************************************************
                          musspikeinput.cpp  -  description
                             -------------------
    begin                : Mon Mar 25 2002
    copyright            : (C) 2002 by Matt Grover, Rdiger Koch
    email                : mpgrover@sourceforge.net, rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "musspikeinput.h"
#include <amygdala/inputneuron.h>
#include <amygdala/network.h>
#include <amygdala/utilities.h>
#include <math.h>

using namespace Amygdala;

MusSpikeInput::MusSpikeInput(const std::string& name):
    SpikeInput(name),
    maxSpikeRate(100.0/1000000.0)
{
    jitter = 2000;   // 2 ms
}

MusSpikeInput::~MusSpikeInput()
{
}

void MusSpikeInput::ReadInputBuffer()
{
}

void MusSpikeInput::TriggerSpikeTrain(InputNeuron* nrn, float t_decay, AmTimeInt t0)
{
    InSpike nextSpike;
    double Ri = maxSpikeRate; // initial rate
    double R = 0.0;   // rate at next spike
    double dt = 0.0;  // time to next spike
    double T = 0.0;   // time of last spike
    double a = -Ri / (double)t_decay;
    int numSpikes;
    int cFactor = 10;

    nextSpike.inNeuron = nrn;
    numSpikes = int(0.5*(Ri*t_decay))+1;
    // increase the number of spikes by a factor of
    // 10 to allow for simulation of a continuous current
    //numSpikes = numSpikes*cFactor;
    T = double(t0);
    for (int i = 0; i<numSpikes; i++) {
        R = sqrt( fabs((Ri*Ri) + (2*a)) );
        dt = (R - Ri)/a;
        for (int j=0; j<cFactor; ++j) {
            double tf = T + dt+(dt/100.0)*Utilities::RandPercent();
            nextSpike.spikeTime = (AmTimeInt)tf;
            inputSpike.push_back(nextSpike);
        }
        T = T + dt;
        Ri = R;
    }
}

void MusSpikeInput::GetReadyForRun(){
   ScheduleQueuedSpikes();
}
