/***************************************************************************
                          musspikeinput.h  -  description
                             -------------------
    begin                : Mon Mar 25 2002
    copyright            : (C) 2002 by Matt Grover, Rdiger Koch
    email                : mpgrover@sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MUSSPIKEINPUT_H
#define MUSSPIKEINPUT_H

#include <amygdala/spikeinput.h>
#include <amygdala/neuron.h>
#include <amygdala/factory.h>
#include <string>

/**
  *@author Matt Grover<matt@amygdala.org>, Rdiger Koch <rkoch@rkoch.org>
  */
namespace Amygdala {

class MusSpikeInput : public SpikeInput  {
public:
    typedef SpikeInputFactory<MusSpikeInput> Factory;
  	MusSpikeInput(const std::string& name);
  	virtual ~MusSpikeInput();

	virtual void AddNeuron(InputNeuron* nrn) { SpikeInput::AddNeuron(nrn); }
  	virtual void ReadInputBuffer();
  	
  	/**
  	 * Trigger a decaying spike train from an area A neuron.
  	 * The train will start off with a spike rate of maxSpikeRate
  	 * and will decay to zero with a rate of decayRate.
  	 */
  	void TriggerSpikeTrain(InputNeuron* nrn, float t_decay, AmTimeInt t0);
  	
  	/**
  	 * Set the maximum spike rate for a spike train.  The default
  	 * will be 200 spikes/sec.
  	 */
  	void SetMaxSpikeRate(float maxRate) { maxSpikeRate = maxRate; }
  	
  	/**
  	 * Return the value of the maximum spike rate.
  	 */
  	float GetMaxSpikeRate() const { return maxSpikeRate; }
    void GetReadyForRun();


private:
    float maxSpikeRate;
    double jitter;
};

namespace Factory {
    static MusSpikeInput::Factory MakeMusSpikeInput;
}
};
#endif
