#include <iostream>
#include <string>
#include <amygdala/network.h>
#include <amygdala/flatlayer.h>
#include <amygdala/fastneuron.h>
#include <amygdala/alphaneuron.h>
#include <amygdala/basicneuron.h>
#include <amygdala/inputneuron.h>
#include <amygdala/spikingneuronproperties.h>
#include <amygdala/physicalproperties.h>
#include <amygdala/synapse.h>
#include <amygdala/nconnector.h>
#include <amygdala/synapseproperties.h>
#include <amygdala/axon.h>
#include <amygdala/scheduledspikeinput.h>
#include <amygdala/basicspikeoutput.h>
#include <amygdala/statictrainer.h>
#include <amygdala/pdeltatrainer.h>

using namespace std;
using namespace Amygdala;

FlatLayer* inLayer;
FlatLayer* outLayer;
PDeltaTrainer* pdelta;

void MakePdelta();
void InitInput();

int main(int argc, char** argv)
{
    cout << "Starting Amygdala API test program.\n";
    Network::Init(argc, argv);

    MakePdelta();
    InitInput();

    // MakePdelta should have created two output groups with input neurons in group 1 and p-perceptron
    // neurons in group 2.
    OutputManager::EnableOutput(false, 1);  // turns output on. OutputManager::DisableOutput(1) turns it off
	OutputManager::EnableOutput(true, 2);

    cout << "Calling Network::InitRun()\n";
    Network::GetNetworkRef()->InitRun();
    Network::GetNetworkRef()->Run((3000*30000)+10000);

    pdelta->PrintWeights();

    Network::Cleanup();
    cout << "Done.\n";
}

void MakePdelta()
{
    inLayer = Factory::MakeFlatLayer("input layer");
    outLayer = Factory::MakeFlatLayer("output layer");
    
    Factory::MakeBasicSpikeOutput("output");

    // set up two input neurons
    InputNeuronProperties inProps;
    InputNeuron* in1 = Factory::MakeInputNeuron(1, inProps, inLayer);
    InputNeuron* in2 = Factory::MakeInputNeuron(2, inProps, inLayer);
    OutputManager::AddNeuronToGroup(in1, 1);
    OutputManager::AddNeuronToGroup(in2, 1);

  	AlphaNeuronProperties aProps;
    aProps.SetMembraneConst(8.0);
    aProps.SetExcitatorySynapseConst(4.0);
    aProps.SetInhibitorySynapseConst(6.0);

    cout << "Building output neurons\n";
    // build a pool of 5 neurons that will make up the parallel perceptron
    for (int i=0; i<5; ++i) {
        AmIdInt id = Network::GetNetworkRef()->GetNewNeuronId();
        AlphaNeuron* nrn = Factory::MakeAlphaNeuron(id, aProps, outLayer);
        OutputManager::AddNeuronToGroup(nrn, 2);    // add to the second output group
    }

    // connect each input neuron to each neuron in the output layer
    StaticSynapseProperties ssp(0.005, 1000);

    for (FlatLayer::iterator itr=inLayer->begin(); itr != inLayer->end(); itr++) {
        for (FlatLayer::iterator oitr=outLayer->begin(); oitr!=outLayer->end(); oitr++) {
            float w = (float)(rand())/(float)(RAND_MAX);
            w = w/2.;
            if (rand()%2) {
                w = w * -1.;    // make half the connections inhibitory
            }
            ssp.SetWeight(w); 
            StaticSynapse::Connect(*itr, dynamic_cast<SpikingNeuron*>(*oitr), ssp);
        }
    }
}

void InitInput()
{
    PDeltaTrainerProperties pdeltaProps(0.005, 0.02, 30000, PDBINARY);
    pdelta = Factory::MakePDeltaTrainer(pdeltaProps, outLayer, "pdelta1");

    // Add an input source to the network and add an input neuron
    ScheduledSpikeInput* si = Factory::MakeScheduledSpikeInput("input");
    si->AddNeuron(dynamic_cast<InputNeuron*>(inLayer->GetNeuron(0)));
    si->AddNeuron(dynamic_cast<InputNeuron*>(inLayer->GetNeuron(1)));

    // Schedule input and training output targets.
    // Each training period is 30ms long and we will go through 3000
    // periods initially.
    // Try out an XOR problem first
    AmTimeInt baseTime=0;
    for (unsigned int i=0; i<1000; ++i) {
        for (AmTimeInt t=1000; t<30000; t+=4000) {
            si->ScheduleSpike(dynamic_cast<InputNeuron*>(inLayer->GetNeuron(0)), t+baseTime);
        }
        // Set trainer to expect output spikes for this period

        for (AmTimeInt t=31000; t<60000; t+=4000) {
            si->ScheduleSpike(dynamic_cast<InputNeuron*>(inLayer->GetNeuron(1)), t+baseTime);
        }
        // Set trainer to expect output spikes for this period

        for (AmTimeInt t=61000; t<90000; t+=8000) {
            si->ScheduleSpike(dynamic_cast<InputNeuron*>(inLayer->GetNeuron(0)), t+baseTime);
            si->ScheduleSpike(dynamic_cast<InputNeuron*>(inLayer->GetNeuron(1)), t+4000+baseTime);
        }
        baseTime+=90000;
    }

    for (unsigned int i=0; i<3000; i+=3) {
        PDeltaTrainer::TrainingExample te1;
        te1.step=i;
        te1.value=1;
        pdelta->AddTrainingExample(te1);

        PDeltaTrainer::TrainingExample te2;
        te2.step=i+1;
        te2.value=1;
        pdelta->AddTrainingExample(te2);

        PDeltaTrainer::TrainingExample te3;
        te3.step=i+2;
        te3.value=0;
        pdelta->AddTrainingExample(te3);
    }
}

