/***************************************************************************
 *   Copyright (C) 2005 by Rüdiger Koch                                    *
 *   rkoch@rkoch.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>
#include <stdexcept>

#include "amygdala/simplenet.h"
#include "amygdala/basicneuron.h"
#include "amygdala/simplenet.h"
#include "amygdala/genome.h"

#include "polephysics.h"
#include "poleinput.h"
#include "poleoutput.h"


using namespace Amygdala;
using namespace std;


#define RUNTIME 50000000


int Usage(const char * progname)
{
    cout << "Usage: " << progname << " [-g] http://gene.server.name" << endl;
    cout << "        -g Generate a genome and send it to the server" << endl << endl;
    return -1;
}

// generate some initial genomes
void GenerateForServer(const char * serverUrl)
{
    cout << "Generating Genome for geneserver " << serverUrl << endl;
    SimpleNet snet("BasicNeuron",5 ,3);
    snet.AddInputNeuron(10, 10, 1);   // negative angle
    snet.AddInputNeuron(50, 20, 2);   // negative angle    
    snet.AddInputNeuron(-40, -10, 3);   // positive angle
    snet.AddInputNeuron(10, -40, 4);   // positive angle
    
    snet.AddInputNeuron(10, 50, 5);   // negative angular velocity
    snet.AddInputNeuron(50, 80, 6);   // positive angular velocity
    
    snet.AddOutputNeuron(100, 160, 10);    // negative force
    snet.AddOutputNeuron(120, 140, 11);    // negative force
    snet.AddOutputNeuron(100, 160, 12);    // negative force
    snet.AddOutputNeuron(20, 45, 13);    // positive force
    snet.AddOutputNeuron(45, 60, 14);    // positive force
    snet.AddOutputNeuron(80, 40, 15);    // positive force
    snet.Upload(serverUrl);
}

int main(int argc, char *argv[])
{
    Network::Init(argc, argv);
    if (argc < 2 || argc > 3 ) return Usage(argv[0]);
    if(strncmp("http://", argv[argc-1], strlen("http://")) != 0 &&
       strncmp("file:/", argv[argc-1], strlen("file:/")) != 0) return Usage(argv[0]);
    if(argc == 3 && strncmp("-g", argv[1], strlen("-g")) != 0) return Usage(argv[0]);
    
    if(strncmp("-g", argv[1], strlen("-g")) == 0) {
        GenerateForServer(argv[2]);
        return EXIT_SUCCESS;
    }
    try {  // plenty of possible exceptions. If one is thrown we exit
        PolePhysics pole;
        pole.Reset(0.2);
        Genome genome(argv[1]);
        
        string inputName("PoleInput"), outputName("PoleOutput");
        PoleInput * si = Factory::MakePoleInput(inputName);
        if(!si) throw runtime_error ("Can't make a PoleInput");
        si->SetPolePhysics(&pole);
        
        PoleOutput * poleOutput = Factory::MakePoleOutput(outputName);
        si->SetPoleOutput(poleOutput);

        SimpleNet morphogenesis("BasicNeuron", 5, 3);
        genome.SetHandler(&morphogenesis);
        genome.Get();
        genome.Parse();

// comment that out if you don't want a spike plot
        OutputManager::EnableOutput(true, 2);
        OutputManager::EnableOutput(true, 3);

        Network::GetNetworkRef()->InitRun();
        Network::GetNetworkRef()->Run(RUNTIME);
        AmTimeInt simTime = Network::GetNetworkRef()->SimTime();
        
// resume the simulation
        pole.Reset(-0.2);
        Network::GetNetworkRef()->Run(simTime + RUNTIME);
        simTime = Network::GetNetworkRef()->SimTime();
        cout << "Runtime: " << simTime << endl;
        genome.Submit((int)(pole.GetScore()*simTime*simTime / (4*float(RUNTIME)*float(RUNTIME)) ) );
    } catch (runtime_error e){
        cout << "Caught Error: " << e.what() << endl;
    }
}
