/***************************************************************************
 *   Copyright (C) 2005 by Rüdiger Koch                                    *
 *   rkoch@rkoch.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <amygdala/network.h>
#include <amygdala/inputneuron.h>
#include "poleinput.h"
#include <iostream>
#include <cmath>

using namespace Amygdala;
using namespace std;

PoleInput::PoleInput()
{
}

PoleInput::PoleInput(const std::string & name) : SpikeInput(name)
{
    memset (nextSchedSpike, 255, sizeof(nextSchedSpike));
}


PoleInput::~PoleInput()
{
}


void PoleInput::ReadInputBuffer()
{
/*  // Test the physix model
    float force = 0;
    int step = 0;
    for (step=0; step < 1000000; step++){ // one second
        const PolePhysics::Rotation * rotation = polePhysics->TimeStep(force);
        force = rotation->phi*30. + rotation->omega*30.;
        if(step%100 == 0) cout << "phi = " << rotation->phi << "   omega = " << rotation->omega << endl; 
        step++;
    }
    exit(0);*/
    float force = poleOutput->GetMotorOutput();
    const PolePhysics::Rotation * rotation = polePhysics->TimeStep(force);
    AmTimeInt simTime = Network::GetNetworkRef()->SimTime();
    if(simTime%100000 == 0)cout << "PHI: " << rotation->phi << endl;
/*    cout << "simtime = " << simTime << "   phi = " << rotation->phi << "     omega = " << rotation->omega << endl;
    cout << "schedule";
    for (int i=0; i<4;i++) cout << "   " << nextSchedSpike[i];
    cout << endl;*/
    
//    schedule inputspikes that are ready to go
    for(AmIdInt id = 1; id <= 6; id++){
        if(nextSchedSpike[id-1] - simTime <= 1000){
  //          cout << "Scheduling Event for id = " << id << endl;
            InputNeuron* inNrn = dynamic_cast <InputNeuron *>(Network::GetNetworkRef()->GetNeuron(id));
            InSpike inSpike;
            inSpike.inNeuron = inNrn;
            inSpike.spikeTime = nextSchedSpike[id-1];
            inputSpike.push_back(inSpike);
            memset (&(nextSchedSpike[id-1]), 255, sizeof(AmTimeInt));
        }
    }
//    cout << "schedule";
//    for (int i=0; i<4;i++) cout << "   " << nextSchedSpike[i];
//    cout << endl;
    ScheduleQueuedSpikes();
    
    float timefactor = 70000.;
    if(rotation->phi < 0.){
       if(nextSchedSpike[0] -simTime > (sqrt(M_PI/4 + rotation->phi) * timefactor))
           nextSchedSpike[0] = (AmTimeInt)(sqrt(M_PI/4 + rotation->phi) * timefactor) + simTime;
    } else {
       if(nextSchedSpike[1] -simTime > sqrt(M_PI/4 - rotation->phi) * timefactor)
           nextSchedSpike[1] = (AmTimeInt)(sqrt(M_PI/4 - rotation->phi) * timefactor)  + simTime;
    }
    timefactor = 50000.;
    if(rotation->phi < 0.){
       if(nextSchedSpike[2] -simTime > (sqrt(M_PI/4 + rotation->phi) * timefactor))
           nextSchedSpike[2] = (AmTimeInt)(sqrt(M_PI/4 + rotation->phi) * timefactor) + simTime;
    } else {
       if(nextSchedSpike[3] -simTime > sqrt(M_PI/4 - rotation->phi) * timefactor)
           nextSchedSpike[3] = (AmTimeInt)(sqrt(M_PI/4 - rotation->phi) * timefactor)  + simTime;
    }
    
    float speedfactor = 100000.;
    if(rotation->omega < 0.){
       if(nextSchedSpike[4] -simTime > -(sqrt(0.3+rotation->omega) * speedfactor))
           nextSchedSpike[4] = -(AmTimeInt)(sqrt(0.3+rotation->omega) * speedfactor) + simTime;
    } else {
       if(nextSchedSpike[5] -simTime > sqrt(0.3-rotation->omega) * speedfactor)
           nextSchedSpike[5] = (AmTimeInt)(sqrt(0.3-rotation->omega) * speedfactor) + simTime;
    }
}

void PoleInput::SetPolePhysics(PolePhysics * pp)
{
    polePhysics = pp;
}

void PoleInput::SetPoleOutput(PoleOutput * po)
{
    poleOutput = po;
}
