/***************************************************************************
 *   Copyright (C) 2005 by Rüdiger Koch                                    *
 *   rkoch@rkoch.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "polephysics.h"

#include <cmath>
#include <iostream>
#include <amygdala/network.h>

using namespace std;
using namespace Amygdala;

PolePhysics::PolePhysics() :
    L(2.), g(9.81), rho(7800.), r(0.02)
{
    m = M_PI*r*r*L*rho;
    I = L*L*m/12 + m*L*L/4;  // Steiner applied to a solid cylinder
    score = 1E8; // start at maximum score
}


PolePhysics::~PolePhysics()
{
}


const PolePhysics::Rotation * PolePhysics::TimeStep(float force)
{
    float dt = ((float)Network::GetNetworkRef()->TimeStepSize()) / 1E6;  // seconds
    float d_omega =  ( L*g*sin(rotation.phi)/2. - force*L*cos(rotation.phi)/2. ) * dt / I;
    rotation.omega += d_omega;
    rotation.phi += rotation.omega*dt;
    V += force * dt / m;
    score -= fabs(V*force);    // the more energy we use the lower the score
    if(fabs(rotation.phi) > M_PI/8) Network::GetNetworkRef()->Stop();
    return &rotation;
}


const float PolePhysics::GetScore()
{
    return score;
}



void PolePhysics::Reset(float phi)
{
    X = 0.;
    V = 0.;
    rotation.phi = phi;
    rotation.omega = 0.;
}
