/***************************************************************************
 *   Copyright (C) 2005 by Rüdiger Koch                                    *
 *   rkoch@rkoch.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef POLEPHYSICS_H
#define POLEPHYSICS_H

/**
Simulates the physics of a 2D pole, balanced on a slider.

@author Rüdiger Koch
*/
class PolePhysics{
public:
    typedef struct { 
        float phi;     // angle of the slider ( 0 = vertical )
        float omega;   // angular velocity of the pole
    } Rotation;
    
    PolePhysics();
    ~PolePhysics();
    const Rotation * TimeStep(float force);
    const float GetScore();
    void Reset(float phi);

private:
    float X;           // postion of the slider
    float V;           // velocity of the slider
    Rotation rotation;      
    float m;           // mass of the pole
    float I;           // moment of inertia relative to the connection point with the slider
    const float L;     // length of the pole
    const float g;     // graviational acceleration
    const float rho;   // density [kg/m^3]
    const float r;     // radius of the pole
    float score;       // always negative.
};

#endif
