/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Mon Feb 25 2002
    copyright            : (C) 2002 by Matt Grover
    email                : mpgrover@sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdexcept>
#include <stdlib.h>
#include <string.h>
#include <amygdala/network.h>
#include "spikeloop.h"

using namespace std;
using namespace Amygdala;

void Usage();
int ParseCommandLine(int argc, char *argv[]);

unsigned int runTime = 1000;
unsigned int layerSize = 6;
unsigned int numLayers = 5;
float sigma = 0.07;

int main(int argc, char *argv[])
{

    // Always the first thing to do: Initialize the network so
    // that Amygdala, Visual, QT or X11 specific arguments
    // can be recognized and stripped. This also creates the
    // Network object which we use below
    try {
    Network::Init(argc, argv);

    // no big deal
    ParseCommandLine(argc, argv);

    // create the spikeloop object. See spikeloop.cpp and
    // spikeloop.h in this directory
    SpikeLoop loop(layerSize, numLayers);
    loop.SetSigma(sigma);
    loop.BuildNetwork();

    // obtain a pointer to The Network object
    Network * theNetwork = Network::GetNetworkRef();
    // This compiles everything into a runnable network
    theNetwork->InitRun();
    // Run the Network for runTime milliseconds
    theNetwork->Run(runTime*1000);
    } catch (runtime_error e){
        cout << e.what() << endl;
        return -1;
    }
}

int ParseCommandLine(int argc, char *argv[])
{
    int index;
    for (index = 1; index < argc; index++)
    {
        // Break out on first parm not starting with a dash
        if (argv[index][0] != '-')
            break;

        if (!strncmp(argv[index], "-t=", 3)) {
            const char* const parm = &argv[index][3];

            // test to make sure parm is numeric
            if (!(runTime = atoi(parm))) {
                cerr << parm << " is not a number.\n";
                return 2;
            }
        }
        else if (!strncmp(argv[index], "-l=", 3)) {
            const char* const parm = &argv[index][3];

            // test to make sure parm is numeric
            if (!(numLayers = atoi(parm))) {
                cerr << parm << " is not a number.\n";
                return 2;
            }
        }
        else if (!strncmp(argv[index], "-s=", 3)) {
            const char* const parm = &argv[index][3];

            // test to make sure parm is numeric
            if (!(layerSize = atoi(parm))) {
                cerr << parm << " is not a number.\n";
                return 2;
            }
        }
        else if (!strncmp(argv[index], "-S=", 3)) {
            const char* parm = &argv[index][3];

            if (!(sigma = atof(parm))) {
                cerr << parm << " is not a valid number.\n";
                return 2;
            }
        }
        else {
            cerr << "Unknown option '" << argv[index]
                 << "', ignoring it\n" << endl;
        }
    }

    return 0;
}

void Usage()
{
    cout <<  "\nUsage: spikeloop [options]\n\n"
             "Options:\n"
             "    -l=n        Number of layers in loop [5]\n"
             "    -s=n        Layer size [6]\n"
             "    -t=n        Simulation run time (ms) [1000]\n"
             "    -S=n        Standard deviation for initial weight distribution [0.01]\n"
             "    --help      Show this help\n\n"
         <<  endl;
}

