/***************************************************************************
                          spikeloop.cpp  -  description
                             -------------------
    begin                : Mon Feb 11 2002
    copyright            : (C) 2002 by Matt Grover
    email                : mgrover@amygdala.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include <iostream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <stdexcept>

#include <amygdala/basicneuron.h>
#include <amygdala/network.h>
#include <amygdala/feedforwardlayer.h>
#include <amygdala/scheduledspikeinput.h>
#include <amygdala/neuronproperties.h>
#include <amygdala/physicalproperties.h>
#include <amygdala/utilities.h>
#include <amygdala/inputneuron.h>
#include <amygdala/inputneuronproperties.h>
#include <amygdala/synapse.h>
#include <amygdala/synapseproperties.h>

#include "spikeloop.h"

using namespace std;
using namespace Amygdala;


SpikeLoop::SpikeLoop(int layerSize, int numLayers):
    poolSize(layerSize),
    numLayers(numLayers)
{
    sigma = 0.0001;
}

SpikeLoop::~SpikeLoop()
{
}

void SpikeLoop::BuildNetwork()
{
    // enable output for the spikeplot
    static const int INPUTGROUP  = 1;
    static const int LOOPGROUP   = 2;
    OutputManager::EnableOutput(true, INPUTGROUP);
    OutputManager::EnableOutput(true, LOOPGROUP);
    
    // start with layer 1 and keep a space with neuron IDs so we can
    // easily see the layers in the spike plot
    unsigned int startId = 1, layerCount = 1;

    // create the factories for our network objects
    /*ScheduledSpikeInput::Factory makeSpikeInput;
    FeedForwardLayer::Factory makeLayer;
    BasicNeuron::Factory makeBasicNeuron;
    InputNeuron::Factory makeInNeuron;*/
    ScheduledSpikeInput* si = Factory::MakeScheduledSpikeInput("input");

    // the input neuron we create below must belong to a Topology
    FeedForwardLayer * inLayer = Factory::MakeFeedForwardLayer("input layer");
    // create one input neuron to trigger the spikeloop
    InputNeuronProperties inProps(true);
    for (AmIdInt id = startId; id < startId + poolSize; id++){
        InputNeuron * inNeuron = Factory::MakeInputNeuron(id, inProps, inLayer);
        if(inNeuron == NULL) throw runtime_error("Creating InputNeuron failed");
        si->AddNeuron(inNeuron);
        si->ScheduleSpike(inNeuron, 1000);
        
        // to get a spikeplot in Visual
        OutputManager::AddNeuronToGroup(inNeuron, INPUTGROUP);
    }
    layers.push_back(inLayer);
    NeuronCoordinates(inLayer, 0, 0);
    
    // these are needed to give the neurons physical coordinates x, y, z
    // not required for spike processing but required for visualization
    const float Radius = 5.0;
    float dPhi = 2 * M_PI / (numLayers);

    // Build the loop layers
    FeedForwardLayer * tmpLayer;
    BasicNeuronProperties bProps(true);
    while (layerCount < numLayers + 1) {
        layerCount++;
        tmpLayer = Factory::MakeFeedForwardLayer(string("looplayer ") + Utilities::itostr(layerCount));
        
        startId += poolSize + 3;  // we want to have a gap in the spikeplot
        for (AmIdInt id = startId; id < startId + poolSize; id++){
            Neuron * neuron = Factory::MakeBasicNeuron(id, bProps, tmpLayer);
            // to get a spikeplot in Visual
            OutputManager::AddNeuronToGroup(neuron, LOOPGROUP);
        }

        layers.push_back(tmpLayer);
        float x = Radius * cos(layerCount*dPhi);
        float y = Radius * sin(layerCount*dPhi);
        NeuronCoordinates(tmpLayer, x, y);
    }

    StaticSynapse::NeuronConnector neuronConnector;
    StaticSynapseProperties sProps(0.5, 1000);

    for (uint i=0; i<layers.size() - 1; i++) {
        layers[i]->ConnectUniform(layers[i+1], neuronConnector, sProps);
    }

    // Connect the last layer to the first hidden layer (since input layers
    // cannot receive connections yet).
    layers[layers.size() - 1]->ConnectUniform(layers[1], neuronConnector, sProps);

}


void SpikeLoop::NeuronCoordinates(FeedForwardLayer *layer, float x, float y){
    float z = -4.;
    for(vector<Neuron*>::iterator it=layer->begin();it!=layer->end(); it++) {
        (*it)->Properties()->GetPhysicalProps()->SetPosition(x, y, z);
        z += 1.5;
    }
}

