#include <iostream>
#include <string>
#include <amygdala/network.h>
#include <amygdala/flatlayer.h>
#include <amygdala/fastneuron.h>
#include <amygdala/alphaneuron.h>
#include <amygdala/basicneuron.h>
#include <amygdala/inputneuron.h>
#include <amygdala/spikingneuronproperties.h>
#include <amygdala/physicalproperties.h>
#include <amygdala/synapse.h>
#include <amygdala/nconnector.h>
#include <amygdala/synapseproperties.h>
#include <amygdala/axon.h>
#include <amygdala/scheduledspikeinput.h>
#include <amygdala/basicspikeoutput.h>
#include <amygdala/statictrainer.h>

using namespace std;
using namespace Amygdala;

void TestAlpha1();
void TestBasic1();
void TestDynamicSynapse();

class TestProps:  public SpikingNeuronProperties {
public:
    TestProps():SpikingNeuronProperties() {}
    TestProps(const TestProps& rhs):SpikingNeuronProperties(rhs) {}
    virtual ~TestProps() {}
    virtual TestProps* Copy() const { return new TestProps(*this); }
};

int main(int argc, char** argv)
{
    cout << "Starting Amygdala API test program.\n";
    Network::Init(argc, argv);
    
    //TestAlpha1();
    TestBasic1();
    //TestDynamicSynapse();
    return 0;
    
    FlatLayer* fl = Factory::MakeFlatLayer("single layer");

    FastNeuronProperties fProps;
    TestProps tProps;
    fProps.SetMembraneConst(10.0);

    AlphaNeuronProperties aProps;
    aProps.SetMembraneConst(10.0);
    aProps.SetExcitatorySynapseConst(6.0);
    aProps.SetInhibitorySynapseConst(2.0);
    
    FastNeuron* nrn = Factory::MakeFastNeuron(1, fProps, fl);  // NeuronFactory is a functor
    //SpikingNeuron* nrn = makeNeuron(1, fProps);
    cout << "Made FastNeuron\n";
    FastNeuron* nrn2 = Factory::MakeFastNeuron(2, fProps, fl);

    FastNeuron* nrn3 = Factory::MakeFastNeuron(3, fProps, fl);
    cout << "FlatLayer name: " << fl->GetName() << endl;
    cout << "NeuronId: " << nrn->GetId() << endl;
    cout << "NeuronId: " << nrn2->GetId() << endl;
    cout << "NeuronId: " << nrn3->GetId() << endl;

    AlphaNeuron* nrn10 = Factory::MakeAlphaNeuron(10, aProps, fl);
    
    //StaticSynapse::NeuronConnector nc;
    StaticSynapseProperties ssp(0.5, 1000);
    StaticSynapse::Connect(nrn, nrn2, ssp);
    StaticSynapse::Connect(nrn, nrn3, ssp);
    
    // Set up an input neuron
    //InputNeuron::Factory makeInNeuron;
    InputNeuronProperties inProps;
    InputNeuron* in1 = Factory::MakeInputNeuron(4, inProps, fl);
    
    ssp.SetWeight(1.0);
    StaticSynapse::Connect(in1, nrn, ssp);
    StaticSynapse::Connect(in1, nrn2, ssp);
    StaticSynapse::Connect(in1, nrn10, ssp);
    
    // Add an input source to the network and add an input neuron
    ScheduledSpikeInput* si = Factory::MakeScheduledSpikeInput("input");
    si->AddNeuron(in1);
    
    // set up some input events
    si->ScheduleSpike(in1, 2000);
    si->ScheduleSpike(in1, 4000);
    si->ScheduleSpike(in1, 5000);
    
    // Setting up output...
    // Each neuron can belong to one or more output groups, but
    // all neurons will report output to the same SpikeOutput object(s).
    // SpikeOutput classes can be built to filter output further if needed.
    // Get rid of the OutputGroup classes -- not any need for them. Call
    // OutputManager::AddNeuronToGroup(nrn*, groupId) to add neurons to
    // a group.  OutputManager will maintain a map of vectors of nrn ptrs
    // keyed by outputId (there are automatically 32 or 64 groups).  The
    // group vector can be retrieved for any group id to determine which
    // neurons are in a given group, or a neuron can be queried to find out
    // which groups it belongs to (it can either return the raw bit field, or
    // it can build a vector of group ids based on the values in the group bit
    // field).  OutputManager will also contain the MakeSpikeOutput() factory
    // function and will own the resulting SpikeOutput object.  Neurons will
    // report output to OutputManager::SendOutput(nrn*) (static), which will pass the
    // message on to all relevant SpikeOutput objects.
    //unsigned short soId = OutputManager::MakeSpikeOutput<BasicSpikeOutput>();
    //SpikeOutput* bso = OutputManager::GetSpikeOutput(soId);	  // returns BasicSpikeOutput*
    BasicSpikeOutput* bso = Factory::MakeBasicSpikeOutput("bso");
    OutputManager::AddNeuronToGroup(nrn3, 1);
    OutputManager::AddNeuronToGroup(nrn, 2);
    OutputManager::AddNeuronToGroup(nrn2, 2);
    OutputManager::AddNeuronToGroup(nrn10, 2);
    OutputManager::AddNeuronToGroup(in1, 3);
    OutputManager::EnableOutput(true, 1);  // turns output on. OutputManager::DisableOutput(1) turns it off
    OutputManager::EnableOutput(true, 2);
    OutputManager::EnableOutput(true, 3);
    
    //OutputManager::EnableOutput(false, 1);
    
    cout << "Calling Network::InitRun()\n";
    Network::GetNetworkRef()->InitRun();
    Network::GetNetworkRef()->Run(10000);
    
    Network::Cleanup();
    cout << "Done.\n";
}

void TestDynamicSynapse()
{
    FlatLayer* fl = Factory::MakeFlatLayer("layer");
    InputNeuronProperties inProps;
    InputNeuron* inNrn = Factory::MakeInputNeuron(1, inProps, fl);
    
    AlphaNeuronProperties alphaProps;
    alphaProps.SetExcitatorySynapseConst(3.0);
    alphaProps.SetInhibitorySynapseConst(6.0);
    alphaProps.SetMembraneConst(30.0);
    AlphaNeuron* alpha = Factory::MakeAlphaNeuron(2, alphaProps, fl);
    
    //DynamicSynapse::NeuronConnector nc;
    DynamicSynapseProperties ssp(1000);
    ssp.SetDelay(1000);
    ssp.SetDynaParam('A', 30./19);
    ssp.SetDynaParam('U', 0.5);
    ssp.SetDynaParam('D', 1.1);
    ssp.SetDynaParam('F', 0.05);
    
    DynamicSynapse::Connect(inNrn, alpha, ssp);
    
    ScheduledSpikeInput* si = Factory::MakeScheduledSpikeInput("input");
    si->AddNeuron(inNrn);
	si->ScheduleSpike(inNrn, 1000);
	si->ScheduleSpike(inNrn, 2000);
	si->ScheduleSpike(inNrn, 10000);
	si->ScheduleSpike(inNrn, 20000);
	si->ScheduleSpike(inNrn, 30000);
	si->ScheduleSpike(inNrn, 50000);
	si->ScheduleSpike(inNrn, 100000);
	si->ScheduleSpike(inNrn, 1e6);
	si->ScheduleSpike(inNrn, 5e6);
    
    Network::GetNetworkRef()->InitRun();
    Network::GetNetworkRef()->Run(6e6);
    
    Network::Cleanup();
    cout << "Done.\n";
}

void TestAlpha1()
{
    FlatLayer* fl = Factory::MakeFlatLayer("single layer");
    
    InputNeuronProperties inProps;
    InputNeuron* in1 = Factory::MakeInputNeuron(1, inProps, fl);
    
    // FIXME: It is possible to create two neurons with the same id.
    // This should raise an exception. (Note here because the next line
    // was making that error.)
    InputNeuron* in2 = Factory::MakeInputNeuron(2, inProps, fl);

  	AlphaNeuronProperties aProps;
    aProps.SetMembraneConst(10.0);
    aProps.SetExcitatorySynapseConst(2.0);
    aProps.SetInhibitorySynapseConst(6.0);
    
    // Build 2 alphaNeurons with each receiving input from both
    // input neurons but with different weights
    
    AlphaNeuron* nrn3 = Factory::MakeAlphaNeuron(3, aProps, fl);
    //AlphaNeuron* nrn4 = makeAlphaNeuron(4, aProps, fl);
    AlphaNeuron* nrn4 = Factory::MakeAlphaNeuron(4, aProps, fl);
    cout << "nrn4 at " << nrn4 << endl;

    StaticSynapseProperties ssp(0.5, 1000);
    
    StaticSynapse::Connect(in1, nrn3, ssp);
    ssp.SetWeight(0.6);
    StaticSynapse::Connect(in2, nrn3, ssp);
    ssp.SetWeight(0.6);
    StaticSynapse::Connect(in1, nrn4, ssp);
    StaticSynapse::Connect(in2, nrn4, ssp);

    // Add an input source to the network and add an input neuron
    ScheduledSpikeInput* si = Factory::MakeScheduledSpikeInput("input");
    si->AddNeuron(in1);
    si->AddNeuron(in2);

    //string outputName = "BasicSpikeOutput";
    //SOFactory * sof =  dynamic_cast<SOFactory*> (FactoryBase::GetRegistry().GetFactory(outputName));
    //unsigned short soId = sof->MakeSpikeOutput("output");
    SpikeOutput* bso = Factory::MakeBasicSpikeOutput("bso");
    OutputManager::AddNeuronToGroup(nrn3, 1);
    OutputManager::AddNeuronToGroup(nrn4, 1);
    OutputManager::AddNeuronToGroup(in1, 2);
    OutputManager::AddNeuronToGroup(in2, 2);

    OutputManager::EnableOutput(true, 1);  // turns output on. OutputManager::DisableOutput(1) turns it off
	OutputManager::EnableOutput(true, 2);

	// Schedule some input
	// Send synchronized spikes from in1 and in2 separated by 4ms
	si->ScheduleSpike(in1, 1000);
	si->ScheduleSpike(in2, 1000);
	si->ScheduleSpike(in1, 6000);
	si->ScheduleSpike(in2, 6000);
	
	// Send input every 4 ms from both input neurons staggered by 2ms
	for (unsigned int i=10000; i<5000000; i+=4000) {
		si->ScheduleSpike(in1, i);
		si->ScheduleSpike(in2, i+2000);
	}
	/*si->ScheduleSpike(in1, 10000);
	si->ScheduleSpike(in2, 12000);
	si->ScheduleSpike(in1, 14000);
	si->ScheduleSpike(in2, 16000);
	si->ScheduleSpike(in1, 18000);
	si->ScheduleSpike(in2, 20000);
	si->ScheduleSpike(in1, 22000);
	si->ScheduleSpike(in2, 24000);
	si->ScheduleSpike(in1, 26000);
	si->ScheduleSpike(in2, 28000);*/
	
	
    cout << "Calling Network::InitRun()\n";
    Network::GetNetworkRef()->InitRun();
    Network::GetNetworkRef()->Run(60000);
    
    Network::Cleanup();
    cout << "Done.\n";
}

void TestBasic1()
{
    FlatLayer* fl = Factory::MakeFlatLayer("single layer");
    
    InputNeuronProperties inProps(1);
    InputNeuron* in1 = Factory::MakeInputNeuron(1, inProps, fl);
    
    inProps.GetPhysicalProps()->SetPosition(0.,0.,1.5);
    InputNeuron* in2 = Factory::MakeInputNeuron(2, inProps, fl);

  	BasicNeuronProperties bProps(1);

    StaticHebbianTrainerProperties tProps;

    bProps.SetSynapseConst(6.0);

    bProps.GetPhysicalProps()->SetPosition(0.,1.5,0.);

    // Build 2 basicNeurons with each receiving input from both
    // input neurons but with different weights
    
    BasicNeuron* nrn3 = Factory::MakeBasicNeuron(3, bProps, fl);
    bProps.GetPhysicalProps()->SetPosition(0.,1.5,1.5);
    BasicNeuron* nrn4 = Factory::MakeBasicNeuron(4, bProps, fl);

    StaticSynapseProperties ssp(0.5, 1000);
    
    StaticSynapse::Connect(in1, nrn3, ssp);
    ssp.SetWeight(0.6);
    StaticSynapse::Connect(in2, nrn3, ssp);
    ssp.SetWeight(0.4);
    ssp.SetDelay(10000);  // increase the synaptic delay to 10ms for nrn4
    StaticSynapse::Connect(in1, nrn4, ssp);
    StaticSynapse::Connect(in2, nrn4, ssp);

    // Add some trainers
    Factory::MakeStaticHebbianTrainer(tProps, nrn3, "hebb1");
    Factory::MakeStaticHebbianTrainer(tProps, nrn4, "hebb2");

    // Add an input source to the network and add an input neuron
    ScheduledSpikeInput* si = Factory::MakeScheduledSpikeInput("input");
    si->AddNeuron(in1);
    si->AddNeuron(in2);
    
    SpikeOutput* bso = Factory::MakeBasicSpikeOutput("output");
    OutputManager::AddNeuronToGroup(nrn3, 1);
    OutputManager::AddNeuronToGroup(nrn4, 1);
    OutputManager::AddNeuronToGroup(in1, 2);
    OutputManager::AddNeuronToGroup(in2, 2);

    OutputManager::EnableOutput(true, 1);  // turns output on. OutputManager::DisableOutput(1) turns it off
	OutputManager::EnableOutput(true, 2);

	// Schedule some input
	// Send synchronized spikes from in1 and in2 separated by 4ms
	si->ScheduleSpike(in1, 2000);
	si->ScheduleSpike(in2, 2000);
	si->ScheduleSpike(in1, 6000);
	si->ScheduleSpike(in2, 6000);
	
	// Send input every 4 ms from both input neurons staggered by 2ms
	for (unsigned int i=10000; i<50000; i+=4000) {
		si->ScheduleSpike(in1, i);
		si->ScheduleSpike(in2, i+2000);
	}
	si->ScheduleSpike(in1, 100000);
	si->ScheduleSpike(in2, 120000);
	si->ScheduleSpike(in1, 140000);
	si->ScheduleSpike(in2, 160000);
	si->ScheduleSpike(in1, 180000);
	si->ScheduleSpike(in2, 200000);
	si->ScheduleSpike(in1, 220000);
	si->ScheduleSpike(in2, 240000);
	si->ScheduleSpike(in1, 260000);
	si->ScheduleSpike(in2, 280000);
	
	
    cout << "Calling Network::InitRun()\n";
    Network::GetNetworkRef()->InitRun();
    Network::GetNetworkRef()->Run(60000);
    
    //Network::GetNetworkRef()->Save("Basic1.xml", false);
    
    Network::Cleanup();
    cout << "Done.\n";
}
