/***************************************************************************
                          gene.cpp  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include "gene.h"
#include "genome.h"
#include "amygdala/logging.h"

Gene::Gene(string _gene){
    LOGGER(6, "Constructing a gene from a string");
    gene = _gene;
}
Gene::Gene(int size){
    LOGGER(6, "Constructing a random gene");
    for (int i=0; i<size;i++){
        gene.push_back((unsigned char)rand());
    }
    gene.push_back((unsigned char)0xFF); // This is the mutation_heat
}

Gene::~Gene(){
    LOGGER(6, "Destroying gene");
}


string Gene::GetGene(){
    return gene;
}



void Gene::MutateFlip(){
    unsigned int pos = (unsigned int)((float)(gene.length() - MUTATION_HEAT)*rand()/(RAND_MAX+1.0));
    unsigned char mask = 1 << ((unsigned int) (8.0*rand()/(RAND_MAX+1.0)));
    if(gene[pos] & mask) gene[pos] &= ~mask;
    else gene[pos] |= mask;
    
    LOGGER(6, "Flip Mutation at position " << pos << " New value: " << (unsigned int)(unsigned char) gene[pos]);
}


void Gene::SetGene(string _gene){
    gene = _gene;
}

void Gene::MutatePM(){
    unsigned int pos = (unsigned int)((float)(gene.length() - MUTATION_HEAT)*rand()/(RAND_MAX+1.));
    unsigned char amount = (unsigned char)atoi(Genome::GetAttribute("amount_mutation_pm").c_str());
    if((100.*rand()/(RAND_MAX+1.0)) < 50.){
        gene[pos] += amount;
        LOGGER(6, "P Mutation at position " << pos << " New value: " << (unsigned int)(unsigned char) gene[pos]);
    } else {
        gene[pos] -= amount;
        LOGGER(6, "M Mutation at position " << pos << " New value: " << (unsigned int)(unsigned char) gene[pos]);
    }
}

float Gene::GetMutationHeat(){
    return ((float)(unsigned char) * gene.rbegin()) / 255.;
}


void Gene::MutateHeat(){
    unsigned int pos = gene.length() - 1;
    if((rand() % 2) == 1){
        if(((unsigned char) gene[pos]) <= 250) gene[pos] += 5;
    } else {
        if(((unsigned char) gene[pos]) >= 5) gene[pos] -= 5;
    }
    LOGGER(6, "Mutated mutation heat. New value: " << (unsigned int)(unsigned char) gene[pos]);
}
