/***************************************************************************
                          genome.h  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Rudiger Koch
    email                : rkoch@rkoch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GENOME_H
#define GENOME_H


#include <string>
#include <string.h>
#include <map>
#include <vector>


class Chromosome;
#define MUTATION_HEAT 1

/**holds the Chomosomes
  *@author Rudiger Koch
  */

class Genome {
public: 
    Genome();
    ~Genome();


// Config stuff
  /**  Print out the current configuration */
  static void PrintConfig();
  /** Load the config file */
  static void LoadConfig();
  /** get a config attribute. The second arg is the string for the result */
  static std::string GetAttribute(std::string);
  /** Load a Genome from a file */
  void Load(int id);
  /** Save this genome */
  void Save();
  /** generate a genome */
  void Generate(int id);
  /** Read property of int id. */
  const int& GetId();
  /** This is the female genome. Crossover with the male results
      in offspring which is returned by this function. */
  Genome* Mate(Genome *male);
  /** Deep copy constructor. */
   Genome(Genome* orig);
   
  /** return a string representation of the chromosomeformat 
      suitable to use as HTTP header:
      "Chromosome-format: {genes,size}{genes,size}...." */
    std::string FormatToString();
    static void MateAll(int *fitest);

    /** remove this Genome from the process */
    void Die();

protected: // Protected attributes


    struct ChromosomeFormat {
        int genes;
        int gene_size;
        unsigned char mutationMode;
    };
    /**  The configuration */
    static std::map <std::string, std::string> config;

    /** Holds the chomosomes of this genome */
    std::vector <Chromosome*> chromosomes;

    int id;
protected: // Protected methods
  /** convert a format string to a format vector */
  static std::vector <ChromosomeFormat> StringToFormat(std::string formatString);
};

#endif
